class EventHandlerList(object,IDisposable):
 """
 Provides a simple list of delegates. This class cannot be inherited.

 

 EventHandlerList()
 """
 def AddHandler(self,key,value):
  """
  AddHandler(self: EventHandlerList,key: object,value: Delegate)

   Adds a delegate to the list.

  

   key: The object that owns the event.

   value: The delegate to add to the list.
  """
  pass
 def AddHandlers(self,listToAddFrom):
  """
  AddHandlers(self: EventHandlerList,listToAddFrom: EventHandlerList)

   Adds a list of delegates to the current list.

  

   listToAddFrom: The list to add.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: EventHandlerList)

   Disposes the delegate list.
  """
  pass
 def RemoveHandler(self,key,value):
  """
  RemoveHandler(self: EventHandlerList,key: object,value: Delegate)

   Removes a delegate from the list.

  

   key: The object that owns the event.

   value: The delegate to remove from the list.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __setitem__(self,*args):
  """ x.__setitem__(i,y) <==> x[i]= """
  pass



class ExpandableObjectConverter(TypeConverter):
 """
 Provides a type converter to convert expandable objects to and from various other representations.

 

 ExpandableObjectConverter()
 """
 def GetProperties(self,*__args):
  """
  GetProperties(self: ExpandableObjectConverter,context: ITypeDescriptorContext,value: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Gets a collection of properties for the type of object specified by the value parameter.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: An System.Object that specifies the type of object to get the properties for.

   attributes: An array of type System.Attribute that will be used as a filter.

   Returns: A System.ComponentModel.PropertyDescriptorCollection with the properties that are exposed for 

    the component,or null if there are no properties.
  """
  pass
 def GetPropertiesSupported(self,context=None):
  """
  GetPropertiesSupported(self: ExpandableObjectConverter,context: ITypeDescriptorContext) -> bool

  

   Gets a value indicating whether this object supports properties using the specified context.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true because System.ComponentModel.TypeConverter.GetProperties(System.Object) should be called 

    to find the properties of this object. This method never returns false.
  """
  pass



class TextBoxRenderer(object):
 """ Provides methods used to render a text box control with visual styles. This class cannot be inherited. """
 @staticmethod
 def DrawTextBox(g,bounds,*__args):
  """
  DrawTextBox(g: Graphics,bounds: Rectangle,textBoxText: str,font: Font,flags: TextFormatFlags,state: TextBoxState)

   Draws a text box control in the specified state and bounds,and with the specified text and text 

    formatting.

  

  

   g: The System.Drawing.Graphics used to draw the text box.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the text box.

   textBoxText: The System.String to draw in the text box.

   font: The System.Drawing.Font to apply to textBoxText.

   flags: A bitwise combination of the System.Windows.Forms.TextFormatFlags values.

   state: One of the System.Windows.Forms.VisualStyles.TextBoxState values that specifies the visual state 

    of the text box.

  

  DrawTextBox(g: Graphics,bounds: Rectangle,textBoxText: str,font: Font,textBounds: Rectangle,flags: TextFormatFlags,state: TextBoxState)

   Draws a text box control in the specified state and bounds,and with the specified text,text 

    bounds,and text formatting.

  

  

   g: The System.Drawing.Graphics used to draw the text box.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the text box.

   textBoxText: The System.String to draw in the text box.

   font: The System.Drawing.Font to apply to textBoxText.

   textBounds: The System.Drawing.Rectangle that specifies the bounds of textBoxText.

   flags: A bitwise combination of the System.Windows.Forms.TextFormatFlags values.

   state: One of the System.Windows.Forms.VisualStyles.TextBoxState values that specifies the visual state 

    of the text box.

  

  DrawTextBox(g: Graphics,bounds: Rectangle,textBoxText: str,font: Font,textBounds: Rectangle,state: TextBoxState)

   Draws a text box control in the specified state and bounds,and with the specified text and text 

    bounds.

  

  

   g: The System.Drawing.Graphics used to draw the text box.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the text box.

   textBoxText: The System.String to draw in the text box.

   font: The System.Drawing.Font to apply to textBoxText.

   textBounds: The System.Drawing.Rectangle that specifies the bounds of textBoxText.

   state: One of the System.Windows.Forms.VisualStyles.TextBoxState values that specifies the visual state 

    of the text box.

  

  DrawTextBox(g: Graphics,bounds: Rectangle,state: TextBoxState)

   Draws a text box control in the specified state and bounds.

  

   g: The System.Drawing.Graphics used to draw the text box.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the text box.

   state: One of the System.Windows.Forms.VisualStyles.TextBoxState values that specifies the visual state 

    of the text box.

  

  DrawTextBox(g: Graphics,bounds: Rectangle,textBoxText: str,font: Font,state: TextBoxState)

   Draws a text box control in the specified state and bounds,and with the specified text.

  

   g: The System.Drawing.Graphics used to draw the text box.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the text box.

   textBoxText: The System.String to draw in the text box.

   font: The System.Drawing.Font to apply to textBoxText.

   state: One of the System.Windows.Forms.VisualStyles.TextBoxState values that specifies the visual state 

    of the text box.
  """
  pass
 IsSupported=True




class ComEventsHelper(object):
 """ Provides methods that enable .NET Framework delegates that handle events to be added and removed from COM objects. """
 @staticmethod
 def Combine(rcw,iid,dispid,d):
  """
  Combine(rcw: object,iid: Guid,dispid: int,d: Delegate)

   Adds a delegate to the invocation list of events originating from a COM object.

  

   rcw: The COM object that triggers the events the caller would like to respond to.

   iid: The identifier of the source interface used by the COM object to trigger events.

   dispid: The dispatch identifier of the method on the source interface.

   d: The delegate to invoke when the COM event is fired.
  """
  pass
 @staticmethod
 def Remove(rcw,iid,dispid,d):
  """
  Remove(rcw: object,iid: Guid,dispid: int,d: Delegate) -> Delegate

  

   Removes a delegate from the invocation list of events originating from a COM object.

  

   rcw: The COM object the delegate is attached to.

   iid: The identifier of the source interface used by the COM object to trigger events.

   dispid: The dispatch identifier of the method on the source interface.

   d: The delegate to remove from the invocation list.

   Returns: The delegate that was removed from the invocation list.
  """
  pass
 __all__=[
  'Combine',
  'Remove',
 ]




class WebHeaderCollection(NameValueCollection,ICollection,IEnumerable,ISerializable,IDeserializationCallback):
 """
 Contains protocol headers associated with a request or response.

 

 WebHeaderCollection()
 """
 def Add(self,*__args):
  """
  Add(self: WebHeaderCollection,header: HttpResponseHeader,value: str)

   Inserts the specified header with the specified value into the collection.

  

   header: The header to add to the collection.

   value: The content of the header.

  Add(self: WebHeaderCollection,header: str)

   Inserts the specified header into the collection.

  

   header: The header to add,with the name and value separated by a colon.

  Add(self: WebHeaderCollection,name: str,value: str)

   Inserts a header with the specified name and value into the collection.

  

   name: The header to add to the collection.

   value: The content of the header.

  Add(self: WebHeaderCollection,header: HttpRequestHeader,value: str)

   Inserts the specified header with the specified value into the collection.

  

   header: The header to add to the collection.

   value: The content of the header.
  """
  pass
 def AddWithoutValidate(self,*args):
  """
  AddWithoutValidate(self: WebHeaderCollection,headerName: str,headerValue: str)

   Inserts a header into the collection without checking whether the header is on the restricted 

    header list.

  

  

   headerName: The header to add to the collection.

   headerValue: The content of the header.
  """
  pass
 def BaseAdd(self,*args):
  """
  BaseAdd(self: NameObjectCollectionBase,name: str,value: object)

   Adds an entry with the specified key and value into the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   name: The System.String key of the entry to add. The key can be null.

   value: The System.Object value of the entry to add. The value can be null.
  """
  pass
 def BaseClear(self,*args):
  """
  BaseClear(self: NameObjectCollectionBase)

   Removes all entries from the System.Collections.Specialized.NameObjectCollectionBase instance.
  """
  pass
 def BaseGet(self,*args):
  """
  BaseGet(self: NameObjectCollectionBase,index: int) -> object

  

   Gets the value of the entry at the specified index of the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   index: The zero-based index of the value to get.

   Returns: An System.Object that represents the value of the entry at the specified index.

  BaseGet(self: NameObjectCollectionBase,name: str) -> object

  

   Gets the value of the first entry with the specified key from the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   name: The System.String key of the entry to get. The key can be null.

   Returns: An System.Object that represents the value of the first entry with the specified key,if found; 

    otherwise,null.
  """
  pass
 def BaseGetAllKeys(self,*args):
  """
  BaseGetAllKeys(self: NameObjectCollectionBase) -> Array[str]

  

   Returns a System.String array that contains all the keys in the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

   Returns: A System.String array that contains all the keys in the 

    System.Collections.Specialized.NameObjectCollectionBase instance.
  """
  pass
 def BaseGetAllValues(self,*args):
  """
  BaseGetAllValues(self: NameObjectCollectionBase,type: Type) -> Array[object]

  

   Returns an array of the specified type that contains all the values in the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   type: A System.Type that represents the type of array to return.

   Returns: An array of the specified type that contains all the values in the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  BaseGetAllValues(self: NameObjectCollectionBase) -> Array[object]

  

   Returns an System.Object array that contains all the values in the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

   Returns: An System.Object array that contains all the values in the 

    System.Collections.Specialized.NameObjectCollectionBase instance.
  """
  pass
 def BaseGetKey(self,*args):
  """
  BaseGetKey(self: NameObjectCollectionBase,index: int) -> str

  

   Gets the key of the entry at the specified index of the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   index: The zero-based index of the key to get.

   Returns: A System.String that represents the key of the entry at the specified index.
  """
  pass
 def BaseHasKeys(self,*args):
  """
  BaseHasKeys(self: NameObjectCollectionBase) -> bool

  

   Gets a value indicating whether the System.Collections.Specialized.NameObjectCollectionBase 

    instance contains entries whose keys are not null.

  

   Returns: true if the System.Collections.Specialized.NameObjectCollectionBase instance contains entries 

    whose keys are not null; otherwise,false.
  """
  pass
 def BaseRemove(self,*args):
  """
  BaseRemove(self: NameObjectCollectionBase,name: str)

   Removes the entries with the specified key from the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   name: The System.String key of the entries to remove. The key can be null.
  """
  pass
 def BaseRemoveAt(self,*args):
  """
  BaseRemoveAt(self: NameObjectCollectionBase,index: int)

   Removes the entry at the specified index of the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   index: The zero-based index of the entry to remove.
  """
  pass
 def BaseSet(self,*args):
  """
  BaseSet(self: NameObjectCollectionBase,index: int,value: object)

   Sets the value of the entry at the specified index of the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   index: The zero-based index of the entry to set.

   value: The System.Object that represents the new value of the entry to set. The value can be null.

  BaseSet(self: NameObjectCollectionBase,name: str,value: object)

   Sets the value of the first entry with the specified key in the 

    System.Collections.Specialized.NameObjectCollectionBase instance,if found; otherwise,adds an 

    entry with the specified key and value into the 

    System.Collections.Specialized.NameObjectCollectionBase instance.

  

  

   name: The System.String key of the entry to set. The key can be null.

   value: The System.Object that represents the new value of the entry to set. The value can be null.
  """
  pass
 def Clear(self):
  """
  Clear(self: WebHeaderCollection)

   Removes all headers from the collection.
  """
  pass
 def Get(self,*__args):
  """
  Get(self: WebHeaderCollection,index: int) -> str

  

   Get the value of a particular header in the collection,specified by an index into the 

    collection.

  

  

   index: The zero-based index of the key to get from the collection.

   Returns: A System.String containing the value of the specified header.

  Get(self: WebHeaderCollection,name: str) -> str

  

   Get the value of a particular header in the collection,specified by the name of the header.

  

   name: The name of the Web header.

   Returns: A System.String holding the value of the specified header.
  """
  pass
 def GetEnumerator(self):
  """
  GetEnumerator(self: WebHeaderCollection) -> IEnumerator

  

   Returns an enumerator that can iterate through the System.Net.WebHeaderCollection instance.

   Returns: An System.Collections.IEnumerator for the System.Net.WebHeaderCollection.
  """
  pass
 def GetKey(self,index):
  """
  GetKey(self: WebHeaderCollection,index: int) -> str

  

   Get the header name at the specified position in the collection.

  

   index: The zero-based index of the key to get from the collection.

   Returns: A System.String holding the header name.
  """
  pass
 def GetObjectData(self,serializationInfo,streamingContext):
  """
  GetObjectData(self: WebHeaderCollection,serializationInfo: SerializationInfo,streamingContext: StreamingContext)

   Populates a System.Runtime.Serialization.SerializationInfo with the data needed to serialize the 

    target object.

  

  

   serializationInfo: The System.Runtime.Serialization.SerializationInfo to populate with data.

   streamingContext: A System.Runtime.Serialization.StreamingContext that specifies the destination for this 

    serialization.
  """
  pass
 def GetValues(self,*__args):
  """
  GetValues(self: WebHeaderCollection,index: int) -> Array[str]

  

   Gets an array of header values stored in the index position of the header collection.

  

   index: The header index to return.

   Returns: An array of header strings.

  GetValues(self: WebHeaderCollection,header: str) -> Array[str]

  

   Gets an array of header values stored in a header.

  

   header: The header to return.

   Returns: An array of header strings.
  """
  pass
 def InvalidateCachedArrays(self,*args):
  """
  InvalidateCachedArrays(self: NameValueCollection)

   Resets the cached arrays of the collection to null.
  """
  pass
 @staticmethod
 def IsRestricted(headerName,response=None):
  """
  IsRestricted(headerName: str,response: bool) -> bool

  

   Tests whether the specified HTTP header can be set for the request or the response.

  

   headerName: The header to test.

   response: Does the Framework test the response or the request?

   Returns: true if the header is restricted; otherwise,false.

  IsRestricted(headerName: str) -> bool

  

   Tests whether the specified HTTP header can be set for the request.

  

   headerName: The header to test.

   Returns: true if the header is restricted; otherwise false.
  """
  pass
 def OnDeserialization(self,sender):
  """
  OnDeserialization(self: WebHeaderCollection,sender: object)

   Implements the System.Runtime.Serialization.ISerializable interface and raises the 

    deserialization event when the deserialization is complete.

  

  

   sender: The source of the deserialization event.
  """
  pass
 def Remove(self,*__args):
  """
  Remove(self: WebHeaderCollection,name: str)

   Removes the specified header from the collection.

  

   name: The name of the header to remove from the collection.

  Remove(self: WebHeaderCollection,header: HttpResponseHeader)

   Removes the specified header from the collection.

  

   header: The System.Net.HttpResponseHeader instance to remove from the collection.

  Remove(self: WebHeaderCollection,header: HttpRequestHeader)

   Removes the specified header from the collection.

  

   header: The System.Net.HttpRequestHeader instance to remove from the collection.
  """
  pass
 def Set(self,*__args):
  """
  Set(self: WebHeaderCollection,header: HttpResponseHeader,value: str)

   Sets the specified header to the specified value.

  

   header: The System.Net.HttpResponseHeader value to set.

   value: The content of the header to set.

  Set(self: WebHeaderCollection,name: str,value: str)

   Sets the specified header to the specified value.

  

   name: The header to set.

   value: The content of the header to set.

  Set(self: WebHeaderCollection,header: HttpRequestHeader,value: str)

   Sets the specified header to the specified value.

  

   header: The System.Net.HttpRequestHeader value to set.

   value: The content of the header to set.
  """
  pass
 def ToByteArray(self):
  """
  ToByteArray(self: WebHeaderCollection) -> Array[Byte]

  

   Converts the System.Net.WebHeaderCollection to a byte array..

   Returns: A System.Byte array holding the header collection.
  """
  pass
 def ToString(self):
  """
  ToString(self: WebHeaderCollection) -> str

  

   This method is obsolete.

   Returns: The System.String representation of the collection.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+y """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 @staticmethod
 def __new__(self):
  """
  __new__(cls: type)

  __new__(cls: type,serializationInfo: SerializationInfo,streamingContext: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __setitem__(self,*args):
  """ x.__setitem__(i,y) <==> x[i]=x.__setitem__(i,y) <==> x[i]= """
  pass
 def __str__(self,*args):
  pass
 AllKeys=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets all header names (keys) in the collection.



Get: AllKeys(self: WebHeaderCollection) -> Array[str]



"""

 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of headers in the collection.



Get: Count(self: WebHeaderCollection) -> int



"""

 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the System.Collections.Specialized.NameObjectCollectionBase instance is read-only.



"""

 Keys=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of header names (keys) in the collection.



Get: Keys(self: WebHeaderCollection) -> KeysCollection



"""





"""Helper functions for working with revit server."""

from collections import namedtuple
from pyrevit.framework import sqlite3

from pyrevit import DB


MODEL_HISTORY_SQLDB = '/Data/Model.db3'


SyncHistory = namedtuple('SyncHistory', ['index',
                                         'userid',
                                         'timestamp'])
"""namedtuple for model sync history data in revit server

Attributes:
    index (int): row index in history db
    userid (str): user identifier
    timestamp (str): time stamp string (e.g. "2017-12-13 19:56:20")
"""


def get_server_path(doc, path_dict):
    """Return file path of a model hosted on revit server.

    Args:
        doc (Document): revit document object
        path_dict (dict): dict of RSN paths and their directory paths

    Examples:
        ```python
        rsn_paths = {'RSN://SERVERNAME': '//servername/filestore'}
        get_server_path(doc, rsn_paths)
        ```
        "//servername/filestore/path/to/model.rvt"
    """
    model_path = doc.GetWorksharingCentralModelPath()
    path = DB.ModelPathUtils.ConvertModelPathToUserVisiblePath(model_path)
    for key, value in path_dict.items():
        path = path.replace(key, value)

    return path


def get_model_sync_history(server_path):
    """Read model sync history from revit server sqlite history file.

    Args:
        server_path (str): directory path of revit server filestore

    Returns:
        (list[SyncHistory]): list of SyncHistory instances

    Examples:
        ```python
        get_model_sync_history("//servername/path/to/model.rvt")
        ```
        [SyncHistory(index=498, userid="user",
                    timestamp="2017-12-13 19:56:20")]
    """
    db_path = server_path + MODEL_HISTORY_SQLDB
    conn = sqlite3.connect(db_path)
    sync_hist = []
    for row in conn.execute("SELECT * FROM ModelHistory"):
        sync_hist.append(SyncHistory(index=int(row[0]),
                                     userid=row[1],
                                     timestamp=row[2][:-1]))
    return sync_hist



class WebBrowserNavigatingEventArgs(CancelEventArgs):
 """
 Provides data for the System.Windows.Forms.WebBrowser.Navigating event.

 

 WebBrowserNavigatingEventArgs(url: Uri,targetFrameName: str)
 """
 @staticmethod
 def __new__(self,url,targetFrameName):
  """ __new__(cls: type,url: Uri,targetFrameName: str) """
  pass
 TargetFrameName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the Web page frame in which the new document will be loaded.



Get: TargetFrameName(self: WebBrowserNavigatingEventArgs) -> str



"""

 Url=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location of the document to which the System.Windows.Forms.WebBrowser control is navigating.



Get: Url(self: WebBrowserNavigatingEventArgs) -> Uri



"""





"""
The main pyRevit test is the startup test which is done by launching Revit.

This module is created to provide a platform to perform complete tests on
different components of pyRevit.
For example, as the git module grows, new tests will be added to the git test
suite to test the full functionality of that module, although only a subset of
functions are used during startup and normal operations of pyRevit.

```python
from unittest import TestCase
class TestWithIndependentOutput(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def doCleanups(self):
        pass
```
"""

import warnings
from pyrevit.unittests.runner import run_module_tests
warnings.filterwarnings("ignore")



class KeyTime(object,IEquatable[KeyTime]):
 """ During the relative course of an animation,a System.Windows.Media.Animation.KeyTime instance specifies the precise timing when a particular key frame should take place. """
 def Equals(self,*__args):
  """
  Equals(self: KeyTime,value: KeyTime) -> bool

  

   Indicates whether this instance is equal to the specified System.Windows.Media.Animation.KeyTime.

  

   value: The object to compare with this instance.

   Returns: true if value is equal to this instance; otherwise,false.

  Equals(keyTime1: KeyTime,keyTime2: KeyTime) -> bool

  

   Indicates whether the two specified System.Windows.Media.Animation.KeyTime structures are equal.

  

   keyTime1: The first value to compare.

   keyTime2: The second value to compare.

   Returns: true if the values of keyTime1 and keyTime2 are equal; otherwise false.

  Equals(self: KeyTime,value: object) -> bool

  

   Indicates whether this instance equals the specified object.

  

   value: The object to compare with this instance.

   Returns: true if value is a System.Windows.Media.Animation.KeyTime that represents the same length of 

    time as this instance; otherwise false.
  """
  pass
 @staticmethod
 def FromPercent(percent):
  """
  FromPercent(percent: float) -> KeyTime

  

   Creates a new System.Windows.Media.Animation.KeyTime instance,with the 

    System.Windows.Media.Animation.KeyTimeType property initialized to the value of the specified 

    parameter.

  

  

   percent: The value of the new System.Windows.Media.Animation.KeyTime.

   Returns: A new System.Windows.Media.Animation.KeyTime instance,initialized to the value of percent.
  """
  pass
 @staticmethod
 def FromTimeSpan(timeSpan):
  """
  FromTimeSpan(timeSpan: TimeSpan) -> KeyTime

  

   Creates a new System.Windows.Media.Animation.KeyTime instance,with the 

    System.Windows.Media.Animation.KeyTimeType property initialized to the value of the specified 

    parameter.

  

  

   timeSpan: The value of the new System.Windows.Media.Animation.KeyTime.

   Returns: A new System.Windows.Media.Animation.KeyTime instance,initialized to the value of timeSpan.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: KeyTime) -> int

  

   Returns an integer hash code representing this instance.

   Returns: An integer hash code.
  """
  pass
 def ToString(self):
  """
  ToString(self: KeyTime) -> str

  

   Returns a string representing this System.Windows.Media.Animation.KeyTime instance.

   Returns: A string representation of this instance.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 Percent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time when the key frame ends expressed as a percentage of the total duration of the animation.



Get: Percent(self: KeyTime) -> float



"""

 TimeSpan=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time when the key frame ends expressed as a time relative to the beginning of the animation.



Get: TimeSpan(self: KeyTime) -> TimeSpan



"""

 Type=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.Animation.KeyTime.Type value this instance represents.



Get: Type(self: KeyTime) -> KeyTimeType



"""


 Paced=None
 Uniform=None




class Region(MarshalByRefObject,IDisposable):
 """
 Describes the interior of a graphics shape composed of rectangles and paths. This class cannot be inherited.

 

 Region()

 Region(rect: Rectangle)

 Region(path: GraphicsPath)

 Region(rect: RectangleF)

 Region(rgnData: RegionData)
 """
 def Clone(self):
  """
  Clone(self: Region) -> Region

  

   Creates an exact copy of this System.Drawing.Region.

   Returns: The System.Drawing.Region that this method creates.
  """
  pass
 def Complement(self,*__args):
  """
  Complement(self: Region,path: GraphicsPath)

   Updates this System.Drawing.Region to contain the portion of the specified 

    System.Drawing.Drawing2D.GraphicsPath that does not intersect with this System.Drawing.Region.

  

  

   path: The System.Drawing.Drawing2D.GraphicsPath to complement this System.Drawing.Region.

  Complement(self: Region,region: Region)

   Updates this System.Drawing.Region to contain the portion of the specified System.Drawing.Region 

    that does not intersect with this System.Drawing.Region.

  

  

   region: The System.Drawing.Region object to complement this System.Drawing.Region object.

  Complement(self: Region,rect: RectangleF)

   Updates this System.Drawing.Region to contain the portion of the specified 

    System.Drawing.RectangleF structure that does not intersect with this System.Drawing.Region.

  

  

   rect: The System.Drawing.RectangleF structure to complement this System.Drawing.Region.

  Complement(self: Region,rect: Rectangle)

   Updates this System.Drawing.Region to contain the portion of the specified 

    System.Drawing.Rectangle structure that does not intersect with this System.Drawing.Region.

  

  

   rect: The System.Drawing.Rectangle structure to complement this System.Drawing.Region.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Region)

   Releases all resources used by this System.Drawing.Region.
  """
  pass
 def Equals(self,*__args):
  """
  Equals(self: Region,region: Region,g: Graphics) -> bool

  

   Tests whether the specified System.Drawing.Region is identical to this System.Drawing.Region on 

    the specified drawing surface.

  

  

   region: The System.Drawing.Region to test.

   g: A System.Drawing.Graphics that represents a drawing surface.

   Returns: true if the interior of region is identical to the interior of this region when the 

    transformation associated with the g parameter is applied; otherwise,false.
  """
  pass
 def Exclude(self,*__args):
  """
  Exclude(self: Region,path: GraphicsPath)

   Updates this System.Drawing.Region to contain only the portion of its interior that does not 

    intersect with the specified System.Drawing.Drawing2D.GraphicsPath.

  

  

   path: The System.Drawing.Drawing2D.GraphicsPath to exclude from this System.Drawing.Region.

  Exclude(self: Region,region: Region)

   Updates this System.Drawing.Region to contain only the portion of its interior that does not 

    intersect with the specified System.Drawing.Region.

  

  

   region: The System.Drawing.Region to exclude from this System.Drawing.Region.

  Exclude(self: Region,rect: Rectangle)

   Updates this System.Drawing.Region to contain only the portion of its interior that does not 

    intersect with the specified System.Drawing.Rectangle structure.

  

  

   rect: The System.Drawing.Rectangle structure to exclude from this System.Drawing.Region.

  Exclude(self: Region,rect: RectangleF)

   Updates this System.Drawing.Region to contain only the portion of its interior that does not 

    intersect with the specified System.Drawing.RectangleF structure.

  

  

   rect: The System.Drawing.RectangleF structure to exclude from this System.Drawing.Region.
  """
  pass
 @staticmethod
 def FromHrgn(hrgn):
  """
  FromHrgn(hrgn: IntPtr) -> Region

  

   Initializes a new System.Drawing.Region from a handle to the specified existing GDI region.

  

   hrgn: A handle to an existing System.Drawing.Region.

   Returns: The new System.Drawing.Region.
  """
  pass
 def GetBounds(self,g):
  """
  GetBounds(self: Region,g: Graphics) -> RectangleF

  

   Gets a System.Drawing.RectangleF structure that represents a rectangle that bounds this 

    System.Drawing.Region on the drawing surface of a System.Drawing.Graphics object.

  

  

   g: The System.Drawing.Graphics on which this System.Drawing.Region is drawn.

   Returns: A System.Drawing.RectangleF structure that represents the bounding rectangle for this 

    System.Drawing.Region on the specified drawing surface.
  """
  pass
 def GetHrgn(self,g):
  """
  GetHrgn(self: Region,g: Graphics) -> IntPtr

  

   Returns a Windows handle to this System.Drawing.Region in the specified graphics context.

  

   g: The System.Drawing.Graphics on which this System.Drawing.Region is drawn.

   Returns: A Windows handle to this System.Drawing.Region.
  """
  pass
 def GetRegionData(self):
  """
  GetRegionData(self: Region) -> RegionData

  

   Returns a System.Drawing.Drawing2D.RegionData that represents the information that describes 

    this System.Drawing.Region.

  

   Returns: A System.Drawing.Drawing2D.RegionData that represents the information that describes this 

    System.Drawing.Region.
  """
  pass
 def GetRegionScans(self,matrix):
  """
  GetRegionScans(self: Region,matrix: Matrix) -> Array[RectangleF]

  

   Returns an array of System.Drawing.RectangleF structures that approximate this 

    System.Drawing.Region after the specified matrix transformation is applied.

  

  

   matrix: A System.Drawing.Drawing2D.Matrix that represents a geometric transformation to apply to the 

    region.

  

   Returns: An array of System.Drawing.RectangleF structures that approximate this System.Drawing.Region 

    after the specified matrix transformation is applied.
  """
  pass
 def Intersect(self,*__args):
  """
  Intersect(self: Region,rect: Rectangle)

   Updates this System.Drawing.Region to the intersection of itself with the specified 

    System.Drawing.Rectangle structure.

  

  

   rect: The System.Drawing.Rectangle structure to intersect with this System.Drawing.Region.

  Intersect(self: Region,path: GraphicsPath)

   Updates this System.Drawing.Region to the intersection of itself with the specified 

    System.Drawing.Drawing2D.GraphicsPath.

  

  

   path: The System.Drawing.Drawing2D.GraphicsPath to intersect with this System.Drawing.Region.

  Intersect(self: Region,region: Region)

   Updates this System.Drawing.Region to the intersection of itself with the specified 

    System.Drawing.Region.

  

  

   region: The System.Drawing.Region to intersect with this System.Drawing.Region.

  Intersect(self: Region,rect: RectangleF)

   Updates this System.Drawing.Region to the intersection of itself with the specified 

    System.Drawing.RectangleF structure.

  

  

   rect: The System.Drawing.RectangleF structure to intersect with this System.Drawing.Region.
  """
  pass
 def IsEmpty(self,g):
  """
  IsEmpty(self: Region,g: Graphics) -> bool

  

   Tests whether this System.Drawing.Region has an empty interior on the specified drawing surface.

  

   g: A System.Drawing.Graphics that represents a drawing surface.

   Returns: true if the interior of this System.Drawing.Region is empty when the transformation associated 

    with g is applied; otherwise,false.
  """
  pass
 def IsInfinite(self,g):
  """
  IsInfinite(self: Region,g: Graphics) -> bool

  

   Tests whether this System.Drawing.Region has an infinite interior on the specified drawing 

    surface.

  

  

   g: A System.Drawing.Graphics that represents a drawing surface.

   Returns: true if the interior of this System.Drawing.Region is infinite when the transformation 

    associated with g is applied; otherwise,false.
  """
  pass
 def IsVisible(self,*__args):
  """
  IsVisible(self: Region,point: Point,g: Graphics) -> bool

  

   Tests whether the specified System.Drawing.Point structure is contained within this 

    System.Drawing.Region when drawn using the specified System.Drawing.Graphics.

  

  

   point: The System.Drawing.Point structure to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,point: Point) -> bool

  

   Tests whether the specified System.Drawing.Point structure is contained within this 

    System.Drawing.Region.

  

  

   point: The System.Drawing.Point structure to test.

   Returns: true when point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,x: int,y: int,g: Graphics) -> bool

  

   Tests whether the specified point is contained within this System.Drawing.Region object when 

    drawn using the specified System.Drawing.Graphics object.

  

  

   x: The x-coordinate of the point to test.

   y: The y-coordinate of the point to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when the specified point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,x: int,y: int,width: int,height: int) -> bool

  

   Tests whether any portion of the specified rectangle is contained within this 

    System.Drawing.Region.

  

  

   x: The x-coordinate of the upper-left corner of the rectangle to test.

   y: The y-coordinate of the upper-left corner of the rectangle to test.

   width: The width of the rectangle to test.

   height: The height of the rectangle to test.

   Returns: true when any portion of the specified rectangle is contained within this System.Drawing.Region; 

    otherwise,false.

  

  IsVisible(self: Region,rect: Rectangle,g: Graphics) -> bool

  

   Tests whether any portion of the specified System.Drawing.Rectangle structure is contained 

    within this System.Drawing.Region when drawn using the specified System.Drawing.Graphics.

  

  

   rect: The System.Drawing.Rectangle structure to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when any portion of the rect is contained within this System.Drawing.Region; otherwise,

    false.

  

  IsVisible(self: Region,x: int,y: int,width: int,height: int,g: Graphics) -> bool

  

   Tests whether any portion of the specified rectangle is contained within this 

    System.Drawing.Region when drawn using the specified System.Drawing.Graphics.

  

  

   x: The x-coordinate of the upper-left corner of the rectangle to test.

   y: The y-coordinate of the upper-left corner of the rectangle to test.

   width: The width of the rectangle to test.

   height: The height of the rectangle to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when any portion of the specified rectangle is contained within this System.Drawing.Region; 

    otherwise,false.

  

  IsVisible(self: Region,rect: Rectangle) -> bool

  

   Tests whether any portion of the specified System.Drawing.Rectangle structure is contained 

    within this System.Drawing.Region.

  

  

   rect: The System.Drawing.Rectangle structure to test.

   Returns: This method returns true when any portion of rect is contained within this 

    System.Drawing.Region; otherwise,false.

  

  IsVisible(self: Region,rect: RectangleF,g: Graphics) -> bool

  

   Tests whether any portion of the specified System.Drawing.RectangleF structure is contained 

    within this System.Drawing.Region when drawn using the specified System.Drawing.Graphics.

  

  

   rect: The System.Drawing.RectangleF structure to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when rect is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,x: Single,y: Single,g: Graphics) -> bool

  

   Tests whether the specified point is contained within this System.Drawing.Region when drawn 

    using the specified System.Drawing.Graphics.

  

  

   x: The x-coordinate of the point to test.

   y: The y-coordinate of the point to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when the specified point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,point: PointF) -> bool

  

   Tests whether the specified System.Drawing.PointF structure is contained within this 

    System.Drawing.Region.

  

  

   point: The System.Drawing.PointF structure to test.

   Returns: true when point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,x: Single,y: Single) -> bool

  

   Tests whether the specified point is contained within this System.Drawing.Region.

  

   x: The x-coordinate of the point to test.

   y: The y-coordinate of the point to test.

   Returns: true when the specified point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,point: PointF,g: Graphics) -> bool

  

   Tests whether the specified System.Drawing.PointF structure is contained within this 

    System.Drawing.Region when drawn using the specified System.Drawing.Graphics.

  

  

   point: The System.Drawing.PointF structure to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when point is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,x: Single,y: Single,width: Single,height: Single,g: Graphics) -> bool

  

   Tests whether any portion of the specified rectangle is contained within this 

    System.Drawing.Region when drawn using the specified System.Drawing.Graphics.

  

  

   x: The x-coordinate of the upper-left corner of the rectangle to test.

   y: The y-coordinate of the upper-left corner of the rectangle to test.

   width: The width of the rectangle to test.

   height: The height of the rectangle to test.

   g: A System.Drawing.Graphics that represents a graphics context.

   Returns: true when any portion of the specified rectangle is contained within this System.Drawing.Region; 

    otherwise,false.

  

  IsVisible(self: Region,rect: RectangleF) -> bool

  

   Tests whether any portion of the specified System.Drawing.RectangleF structure is contained 

    within this System.Drawing.Region.

  

  

   rect: The System.Drawing.RectangleF structure to test.

   Returns: true when any portion of rect is contained within this System.Drawing.Region; otherwise,false.

  IsVisible(self: Region,x: Single,y: Single,width: Single,height: Single) -> bool

  

   Tests whether any portion of the specified rectangle is contained within this 

    System.Drawing.Region.

  

  

   x: The x-coordinate of the upper-left corner of the rectangle to test.

   y: The y-coordinate of the upper-left corner of the rectangle to test.

   width: The width of the rectangle to test.

   height: The height of the rectangle to test.

   Returns: true when any portion of the specified rectangle is contained within this System.Drawing.Region 

    object; otherwise,false.
  """
  pass
 def MakeEmpty(self):
  """
  MakeEmpty(self: Region)

   Initializes this System.Drawing.Region to an empty interior.
  """
  pass
 def MakeInfinite(self):
  """
  MakeInfinite(self: Region)

   Initializes this System.Drawing.Region object to an infinite interior.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ReleaseHrgn(self,regionHandle):
  """
  ReleaseHrgn(self: Region,regionHandle: IntPtr)

   Releases the handle of the System.Drawing.Region.

  

   regionHandle: The handle to the System.Drawing.Region.
  """
  pass
 def Transform(self,matrix):
  """
  Transform(self: Region,matrix: Matrix)

   Transforms this System.Drawing.Region by the specified System.Drawing.Drawing2D.Matrix.

  

   matrix: The System.Drawing.Drawing2D.Matrix by which to transform this System.Drawing.Region.
  """
  pass
 def Translate(self,dx,dy):
  """
  Translate(self: Region,dx: int,dy: int)

   Offsets the coordinates of this System.Drawing.Region by the specified amount.

  

   dx: The amount to offset this System.Drawing.Region horizontally.

   dy: The amount to offset this System.Drawing.Region vertically.

  Translate(self: Region,dx: Single,dy: Single)

   Offsets the coordinates of this System.Drawing.Region by the specified amount.

  

   dx: The amount to offset this System.Drawing.Region horizontally.

   dy: The amount to offset this System.Drawing.Region vertically.
  """
  pass
 def Union(self,*__args):
  """
  Union(self: Region,path: GraphicsPath)

   Updates this System.Drawing.Region to the union of itself and the specified 

    System.Drawing.Drawing2D.GraphicsPath.

  

  

   path: The System.Drawing.Drawing2D.GraphicsPath to unite with this System.Drawing.Region.

  Union(self: Region,region: Region)

   Updates this System.Drawing.Region to the union of itself and the specified 

    System.Drawing.Region.

  

  

   region: The System.Drawing.Region to unite with this System.Drawing.Region.

  Union(self: Region,rect: Rectangle)

   Updates this System.Drawing.Region to the union of itself and the specified 

    System.Drawing.Rectangle structure.

  

  

   rect: The System.Drawing.Rectangle structure to unite with this System.Drawing.Region.

  Union(self: Region,rect: RectangleF)

   Updates this System.Drawing.Region to the union of itself and the specified 

    System.Drawing.RectangleF structure.

  

  

   rect: The System.Drawing.RectangleF structure to unite with this System.Drawing.Region.
  """
  pass
 def Xor(self,*__args):
  """
  Xor(self: Region,path: GraphicsPath)

   Updates this System.Drawing.Region to the union minus the intersection of itself with the 

    specified System.Drawing.Drawing2D.GraphicsPath.

  

  

   path: The System.Drawing.Drawing2D.GraphicsPath to erload:System.Drawing.Region.Xor with this 

    System.Drawing.Region.

  

  Xor(self: Region,region: Region)

   Updates this System.Drawing.Region to the union minus the intersection of itself with the 

    specified System.Drawing.Region.

  

  

   region: The System.Drawing.Region to erload:System.Drawing.Region.Xor with this System.Drawing.Region.

  Xor(self: Region,rect: RectangleF)

   Updates this System.Drawing.Region to the union minus the intersection of itself with the 

    specified System.Drawing.RectangleF structure.

  

  

   rect: The System.Drawing.RectangleF structure to 

    System.Drawing.Region.Xor(System.Drawing.Drawing2D.GraphicsPath) with this 

    System.Drawing.Region.

  

  Xor(self: Region,rect: Rectangle)

   Updates this System.Drawing.Region to the union minus the intersection of itself with the 

    specified System.Drawing.Rectangle structure.

  

  

   rect: The System.Drawing.Rectangle structure to erload:System.Drawing.Region.Xor with this 

    System.Drawing.Region.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,rect: RectangleF)

  __new__(cls: type,rect: Rectangle)

  __new__(cls: type,path: GraphicsPath)

  __new__(cls: type,rgnData: RegionData)
  """
  pass



class ICredentialPolicy:
 """ Defines the credential policy to be used for resource requests that are made using System.Net.WebRequest and its derived classes. """
 def ShouldSendCredential(self,challengeUri,request,credential,authenticationModule):
  """
  ShouldSendCredential(self: ICredentialPolicy,challengeUri: Uri,request: WebRequest,credential: NetworkCredential,authenticationModule: IAuthenticationModule) -> bool

  

   Returns a System.Boolean that indicates whether the client's credentials are sent with a 

    resource request made using an instance of the System.Net.WebRequest class.

  

  

   challengeUri: The System.Uri that will receive the request. For more information,see the Remarks section.

   request: The System.Net.WebRequest that represents the resource being requested.

   credential: The System.Net.NetworkCredential that will be sent with the request if this method returns true.

   authenticationModule: The System.Net.IAuthenticationModule that will conduct the authentication,if authentication is 

    required.

  

   Returns: true if the credentials are sent with the request; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



""" Curve Wrappers """

from math import pi as PI

from rpw import revit, DB
from rpw.base import BaseObjectWrapper
from rpw.db.element import Element
from rpw.db.xyz import XYZ
from rpw.utils.mixins import ByNameCollectMixin


class Curve(BaseObjectWrapper):
    """
    DB.Curve Wrapper

    >>> curve = Curve.new(ExistingCurveObject)
    >>> curve.create_detail()

    """

    _revit_object_class = DB.Curve

    def create_detail(self, view=None, doc=revit.doc):
        """
        Args:
            view (``DB.View``): Optional View. Default: ``uidoc.ActiveView``
            doc (``DB.Document``): Optional Document. Default: ``doc``
        """
        # TODO: Accept Detail Type (GraphicStyle)
        view = view or revit.active_view.unwrap()
        return doc.Create.NewDetailCurve(view, self._revit_object)

    def create_model(self, view=None, doc=revit.doc):
        # http://www.revitapidocs.com/2017.1/b880c4d7-9841-e44e-2a1c-36fefe274e2e.htm
        raise NotImplemented


class Line(Curve):

    """
    DB.Line Wrapper

    >>> line = Line.new([-10,0], [10,0])
    >>> # or
    >>> line = Line.new(ExistingLineObject)
    >>> line.create_detail()

    """
    _revit_object_class = DB.Line

    @classmethod
    def new(cls, pt1, pt2):
        """
        Args:
            point1 (``point``): Point like object. See :any:`XYZ`
            point2 (``point``): Point like object. See :any:`XYZ`
        """
        pt1 = XYZ(pt1)
        pt2 = XYZ(pt2)
        line = DB.Line.CreateBound(pt1.unwrap(), pt2.unwrap())
        return cls(line)

    @property
    def start_point(self):
        """ Start Point of line """
        return XYZ(self._revit_object.GetEndPoint(0))

    @property
    def end_point(self):
        """ End Point of line """
        return XYZ(self._revit_object.GetEndPoint(1))

    @property
    def mid_point(self):
        """ Mid Point of line """
        return XYZ(self._revit_object.GetEndPoint(0.5))

    @property
    def end_points(self):
        """ End Points of line """
        return (XYZ(self.start_point), XYZ(self.end_point))


class Ellipse(Curve):
    """

    >>> ellipse = Ellipse.new([-10,0], [10,0])
    >>> # or
    >>> ellipse = Ellipse.new(ExistingEllipseObject)
    >>> ellipse.create_detail()

    """
    _revit_object_class = DB.Ellipse

    @classmethod
    def new(cls, center, x_radius, y_radius, x_axis=None, y_axis=None,
            start_param=0.0, end_param=2*PI):
        """
        Args:
            center (``point``): Center of Ellipse
            x_radius (``float``): X Radius
            y_radius (``float``): Y Radius
            x_axis (``point``): X Axis
            y_axis (``point``): Y Axis
            start_param (``float``): Start Parameter
            end_param (``float``): End Parameter
        """
        center = XYZ(center).unwrap()
        x_axis = DB.XYZ(1,0,0) if x_axis is None else XYZ(x_axis).unwrap().Normalize()
        y_axis = DB.XYZ(0,1,0) if y_axis is None else XYZ(y_axis).unwrap().Normalize()

        start_param = start_param or 0.0
        end_param = start_param or PI*2

        ellipse = DB.Ellipse.Create(center, x_radius, y_radius, x_axis, y_axis, start_param, end_param)
        return cls(ellipse)

class Circle(Ellipse):
    """

    >>> circle = Circle.new([-10,0], 2)
    >>> # or
    >>> circle = Circle.new(ExistingCircleObject)
    >>> circle.create_detail()

    """

    @classmethod
    def new(cls, center,
            radius,
            x_axis=None, y_axis=None,
            start_param=0.0, end_param=2*PI):
        """
        Args:
            center (``point``): Center of Ellipse
            x_radius (``float``): X Radius
            x_axis (``point``): X Axis
            y_axis (``point``): Y Axis
            start_param (``float``): Start Parameter
            end_param (``float``): End Parameter
        """
        center = XYZ(center).unwrap()
        x_axis = DB.XYZ(1,0,0) if x_axis is None else XYZ(x_axis).unwrap().Normalize()
        y_axis = DB.XYZ(0,1,0) if y_axis is None else XYZ(y_axis).unwrap().Normalize()

        start_param = start_param or 0.0
        end_param = start_param or PI*2

        circle = DB.Ellipse.Create(center, radius, radius, x_axis, y_axis, start_param, end_param)
        return cls(circle)


class Arc(Curve):
    """

    >>> arc = Arc.new([0,0], [0,0], [0,0])
    >>> # or
    >>> arc = Arc.new(ExistingArcObject)
    >>> arc.create_detail()

    """

    @classmethod
    def new(cls, *args):
            # http://www.revitapidocs.com/2017.1/19c3ba08-5443-c9d4-3a3f-0e78901fe6d4.htm
            # XYZ, XYZ, XYZ
            # Plane, Double, Double, Double (Plane, Radius, startAngle, endAngle)
            # XYZ, Double, Double, Double ,XYZ, XYZ (Center, radius, vectors, angles)
        """
        Args:
            start_pt (``point``): Start Point
            mid_pt (``point``): End Point
            end_pt (``point``): Mid Point
        """
        if len(args) == 3:
            start_pt, end_pt, mid_pt = [XYZ(pt).unwrap() for pt in args]
            arc = DB.Arc.Create(start_pt, end_pt, mid_pt)
        else:
            raise NotImplemented('only arc by 3 pts available')
        return cls(arc)



class RegistryRights(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the access control rights that can be applied to registry objects.

 

 enum (flags) RegistryRights,values: ChangePermissions (262144),CreateLink (32),CreateSubKey (4),Delete (65536),EnumerateSubKeys (8),ExecuteKey (131097),FullControl (983103),Notify (16),QueryValues (1),ReadKey (131097),ReadPermissions (131072),SetValue (2),TakeOwnership (524288),WriteKey (131078)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 ChangePermissions=None
 CreateLink=None
 CreateSubKey=None
 Delete=None
 EnumerateSubKeys=None
 ExecuteKey=None
 FullControl=None
 Notify=None
 QueryValues=None
 ReadKey=None
 ReadPermissions=None
 SetValue=None
 TakeOwnership=None
 value__=None
 WriteKey=None




class UCOMIMoniker:
 """ Use System.Runtime.InteropServices.ComTypes.IMoniker instead. """
 def BindToObject(self,pbc,pmkToLeft,riidResult,ppvResult):
  """
  BindToObject(self: UCOMIMoniker,pbc: UCOMIBindCtx,pmkToLeft: UCOMIMoniker,riidResult: Guid) -> (Guid,object)

  

   Uses the moniker to bind to the object it identifies.

  

   pbc: A reference to the IBindCtx interface on the bind context object used in this binding operation.

   pmkToLeft: A reference to the moniker to the left of this moniker,if the moniker is part of a composite 

    moniker.

  

   riidResult: The interface identifier (IID) of the interface the client intends to use to communicate with 

    the object that the moniker identifies.
  """
  pass
 def BindToStorage(self,pbc,pmkToLeft,riid,ppvObj):
  """
  BindToStorage(self: UCOMIMoniker,pbc: UCOMIBindCtx,pmkToLeft: UCOMIMoniker,riid: Guid) -> (Guid,object)

  

   Retrieves an interface pointer to the storage that contains the object identified by the moniker.

  

   pbc: A reference to the IBindCtx interface on the bind context object used during this binding 

    operation.

  

   pmkToLeft: A reference to the moniker to the left of this moniker,if the moniker is part of a composite 

    moniker.

  

   riid: The interface identifier (IID) of the storage interface requested.
  """
  pass
 def CommonPrefixWith(self,pmkOther,ppmkPrefix):
  """
  CommonPrefixWith(self: UCOMIMoniker,pmkOther: UCOMIMoniker) -> UCOMIMoniker

  

   Creates a new moniker based on the common prefix that this moniker shares with another moniker.

  

   pmkOther: A reference to the IMoniker interface on another moniker to compare with this for a common 

    prefix.
  """
  pass
 def ComposeWith(self,pmkRight,fOnlyIfNotGeneric,ppmkComposite):
  """
  ComposeWith(self: UCOMIMoniker,pmkRight: UCOMIMoniker,fOnlyIfNotGeneric: bool) -> UCOMIMoniker

  

   Combines the current moniker with another moniker,creating a new composite moniker.

  

   pmkRight: A reference to the IMoniker interface on the moniker to compose onto the end of this moniker.

   fOnlyIfNotGeneric: If true,the caller requires a nongeneric composition,so the operation proceeds only if 

    pmkRight is a moniker class that this moniker can compose with in some way other than forming a 

    generic composite. If false,the method can create a generic composite if necessary.
  """
  pass
 def Enum(self,fForward,ppenumMoniker):
  """
  Enum(self: UCOMIMoniker,fForward: bool) -> UCOMIEnumMoniker

  

   Supplies a pointer to an enumerator that can enumerate the components of a composite moniker.

  

   fForward: If true,enumerates the monikers from left to right. If false,enumerates from right to left.
  """
  pass
 def GetClassID(self,pClassID):
  """
  GetClassID(self: UCOMIMoniker) -> Guid

  

   Retrieves the class identifier (CLSID) of an object.
  """
  pass
 def GetDisplayName(self,pbc,pmkToLeft,ppszDisplayName):
  """
  GetDisplayName(self: UCOMIMoniker,pbc: UCOMIBindCtx,pmkToLeft: UCOMIMoniker) -> str

  

   Gets the display name,which is a user-readable representation of this moniker.

  

   pbc: A reference to the bind context to use in this operation.

   pmkToLeft: A reference to the moniker to the left of this moniker,if the moniker is part of a composite 

    moniker.
  """
  pass
 def GetSizeMax(self,pcbSize):
  """
  GetSizeMax(self: UCOMIMoniker) -> Int64

  

   Returns the size in bytes of the stream needed to save the object.
  """
  pass
 def GetTimeOfLastChange(self,pbc,pmkToLeft,pFileTime):
  """
  GetTimeOfLastChange(self: UCOMIMoniker,pbc: UCOMIBindCtx,pmkToLeft: UCOMIMoniker) -> FILETIME

  

   Provides a number representing the time the object identified by this moniker was last changed.

  

   pbc: A reference to the bind context to be used in this binding operation.

   pmkToLeft: A reference to the moniker to the left of this moniker,if the moniker is part of a composite 

    moniker.
  """
  pass
 def Hash(self,pdwHash):
  """
  Hash(self: UCOMIMoniker) -> int

  

   Calculates a 32-bit integer using the internal state of the moniker.
  """
  pass
 def Inverse(self,ppmk):
  """
  Inverse(self: UCOMIMoniker) -> UCOMIMoniker

  

   Provides a moniker that,when composed to the right of this moniker or one of similar structure,

    composes to nothing.
  """
  pass
 def IsDirty(self):
  """
  IsDirty(self: UCOMIMoniker) -> int

  

   Checks the object for changes since it was last saved.

   Returns: An S_OKHRESULT value if the object has changed; otherwise,an S_FALSEHRESULT value.
  """
  pass
 def IsEqual(self,pmkOtherMoniker):
  """
  IsEqual(self: UCOMIMoniker,pmkOtherMoniker: UCOMIMoniker)

   Compares this moniker with a specified moniker and indicates whether they are identical.

  

   pmkOtherMoniker: A reference to the moniker to be used for comparison.
  """
  pass
 def IsRunning(self,pbc,pmkToLeft,pmkNewlyRunning):
  """
  IsRunning(self: UCOMIMoniker,pbc: UCOMIBindCtx,pmkToLeft: UCOMIMoniker,pmkNewlyRunning: UCOMIMoniker)

   Determines whether the object that is identified by this moniker is currently loaded and running.

  

   pbc: A reference to the bind context to be used in this binding operation.

   pmkToLeft: A reference to the moniker to the left of this moniker if this moniker is part of a composite.

   pmkNewlyRunning: A reference to the moniker most recently added to the Running Object Table.
  """
  pass
 def IsSystemMoniker(self,pdwMksys):
  """
  IsSystemMoniker(self: UCOMIMoniker) -> int

  

   Indicates whether this moniker is of one of the system-supplied moniker classes.
  """
  pass
 def Load(self,pStm):
  """
  Load(self: UCOMIMoniker,pStm: UCOMIStream)

   Initializes an object from the stream where it was previously saved.

  

   pStm: Stream from which the object is loaded.
  """
  pass
 def ParseDisplayName(self,pbc,pmkToLeft,pszDisplayName,pchEaten,ppmkOut):
  """
  ParseDisplayName(self: UCOMIMoniker,pbc: UCOMIBindCtx,pmkToLeft: UCOMIMoniker,pszDisplayName: str) -> (int,UCOMIMoniker)

  

   Reads as many characters of the specified display name as it understands and builds a moniker 

    corresponding to the portion read.

  

  

   pbc: A reference to the bind context to be used in this binding operation.

   pmkToLeft: A reference to the moniker that has been built out of the display name up to this point.

   pszDisplayName: A reference to the string containing the remaining display name to parse.
  """
  pass
 def Reduce(self,pbc,dwReduceHowFar,ppmkToLeft,ppmkReduced):
  """
  Reduce(self: UCOMIMoniker,pbc: UCOMIBindCtx,dwReduceHowFar: int,ppmkToLeft: UCOMIMoniker) -> (UCOMIMoniker,UCOMIMoniker)

  

   Returns a reduced moniker which is another moniker that refers to the same object as this 

    moniker but can be bound with equal or greater efficiency.

  

  

   pbc: A reference to the IBindCtx interface on the bind context to be used in this binding operation.

   dwReduceHowFar: Specifies how far this moniker should be reduced.

   ppmkToLeft: A reference to the moniker to the left of this moniker.
  """
  pass
 def RelativePathTo(self,pmkOther,ppmkRelPath):
  """
  RelativePathTo(self: UCOMIMoniker,pmkOther: UCOMIMoniker) -> UCOMIMoniker

  

   Supplies a moniker that,when appended to this moniker (or one with a similar structure),yields 

    the specified moniker.

  

  

   pmkOther: A reference to the moniker to which a relative path should be taken.
  """
  pass
 def Save(self,pStm,fClearDirty):
  """
  Save(self: UCOMIMoniker,pStm: UCOMIStream,fClearDirty: bool)

   Saves an object to the specified stream.

  

   pStm: The stream into which the object is saved.

   fClearDirty: Indicates whether to clear the modified flag after the save is complete.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class IConvertible:
 """ Defines methods that convert the value of the implementing reference or value type to a common language runtime type that has an equivalent value. """
 def GetTypeCode(self):
  """
  GetTypeCode(self: IConvertible) -> TypeCode

  

   Returns the System.TypeCode for this instance.

   Returns: The enumerated constant that is the System.TypeCode of the class or value type that implements 

    this interface.
  """
  pass
 def ToBoolean(self,provider):
  """
  ToBoolean(self: IConvertible,provider: IFormatProvider) -> bool

  

   Converts the value of this instance to an equivalent Boolean value using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A Boolean value equivalent to the value of this instance.
  """
  pass
 def ToByte(self,provider):
  """
  ToByte(self: IConvertible,provider: IFormatProvider) -> Byte

  

   Converts the value of this instance to an equivalent 8-bit unsigned integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 8-bit unsigned integer equivalent to the value of this instance.
  """
  pass
 def ToChar(self,provider):
  """
  ToChar(self: IConvertible,provider: IFormatProvider) -> Char

  

   Converts the value of this instance to an equivalent Unicode character using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A Unicode character equivalent to the value of this instance.
  """
  pass
 def ToDateTime(self,provider):
  """
  ToDateTime(self: IConvertible,provider: IFormatProvider) -> DateTime

  

   Converts the value of this instance to an equivalent System.DateTime using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A System.DateTime instance equivalent to the value of this instance.
  """
  pass
 def ToDecimal(self,provider):
  """
  ToDecimal(self: IConvertible,provider: IFormatProvider) -> Decimal

  

   Converts the value of this instance to an equivalent System.Decimal number using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A System.Decimal number equivalent to the value of this instance.
  """
  pass
 def ToDouble(self,provider):
  """
  ToDouble(self: IConvertible,provider: IFormatProvider) -> float

  

   Converts the value of this instance to an equivalent double-precision floating-point number 

    using the specified culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A double-precision floating-point number equivalent to the value of this instance.
  """
  pass
 def ToInt16(self,provider):
  """
  ToInt16(self: IConvertible,provider: IFormatProvider) -> Int16

  

   Converts the value of this instance to an equivalent 16-bit signed integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 16-bit signed integer equivalent to the value of this instance.
  """
  pass
 def ToInt32(self,provider):
  """
  ToInt32(self: IConvertible,provider: IFormatProvider) -> int

  

   Converts the value of this instance to an equivalent 32-bit signed integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 32-bit signed integer equivalent to the value of this instance.
  """
  pass
 def ToInt64(self,provider):
  """
  ToInt64(self: IConvertible,provider: IFormatProvider) -> Int64

  

   Converts the value of this instance to an equivalent 64-bit signed integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 64-bit signed integer equivalent to the value of this instance.
  """
  pass
 def ToSByte(self,provider):
  """
  ToSByte(self: IConvertible,provider: IFormatProvider) -> SByte

  

   Converts the value of this instance to an equivalent 8-bit signed integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 8-bit signed integer equivalent to the value of this instance.
  """
  pass
 def ToSingle(self,provider):
  """
  ToSingle(self: IConvertible,provider: IFormatProvider) -> Single

  

   Converts the value of this instance to an equivalent single-precision floating-point number 

    using the specified culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A single-precision floating-point number equivalent to the value of this instance.
  """
  pass
 def ToString(self,provider):
  """
  ToString(self: IConvertible,provider: IFormatProvider) -> str

  

   Converts the value of this instance to an equivalent System.String using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: A System.String instance equivalent to the value of this instance.
  """
  pass
 def ToType(self,conversionType,provider):
  """
  ToType(self: IConvertible,conversionType: Type,provider: IFormatProvider) -> object

  

   Converts the value of this instance to an System.Object of the specified System.Type that has an 

    equivalent value,using the specified culture-specific formatting information.

  

  

   conversionType: The System.Type to which the value of this instance is converted.

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An System.Object instance of type conversionType whose value is equivalent to the value of this 

    instance.
  """
  pass
 def ToUInt16(self,provider):
  """
  ToUInt16(self: IConvertible,provider: IFormatProvider) -> UInt16

  

   Converts the value of this instance to an equivalent 16-bit unsigned integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 16-bit unsigned integer equivalent to the value of this instance.
  """
  pass
 def ToUInt32(self,provider):
  """
  ToUInt32(self: IConvertible,provider: IFormatProvider) -> UInt32

  

   Converts the value of this instance to an equivalent 32-bit unsigned integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 32-bit unsigned integer equivalent to the value of this instance.
  """
  pass
 def ToUInt64(self,provider):
  """
  ToUInt64(self: IConvertible,provider: IFormatProvider) -> UInt64

  

   Converts the value of this instance to an equivalent 64-bit unsigned integer using the specified 

    culture-specific formatting information.

  

  

   provider: An System.IFormatProvider interface implementation that supplies culture-specific formatting 

    information.

  

   Returns: An 64-bit unsigned integer equivalent to the value of this instance.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class ToolStripItemClickedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ToolStrip.ItemClicked event.

 

 ToolStripItemClickedEventArgs(clickedItem: ToolStripItem)
 """
 @staticmethod
 def __new__(self,clickedItem):
  """ __new__(cls: type,clickedItem: ToolStripItem) """
  pass
 ClickedItem=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the item that was clicked on the System.Windows.Forms.ToolStrip.



Get: ClickedItem(self: ToolStripItemClickedEventArgs) -> ToolStripItem



"""





class PointF(object):
 """
 Represents an ordered pair of floating-point x- and y-coordinates that defines a point in a two-dimensional plane.

 

 PointF(x: Single,y: Single)
 """
 @staticmethod
 def Add(pt,sz):
  """
  Add(pt: PointF,sz: SizeF) -> PointF

  

   Translates a given System.Drawing.PointF by a specified System.Drawing.SizeF.

  

   pt: The System.Drawing.PointF to translate.

   sz: The System.Drawing.SizeF that specifies the numbers to add to the coordinates of pt.

   Returns: The translated System.Drawing.PointF.

  Add(pt: PointF,sz: Size) -> PointF

  

   Translates a given System.Drawing.PointF by the specified System.Drawing.Size.

  

   pt: The System.Drawing.PointF to translate.

   sz: The System.Drawing.Size that specifies the numbers to add to the coordinates of pt.

   Returns: The translated System.Drawing.PointF.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: PointF,obj: object) -> bool

  

   Specifies whether this System.Drawing.PointF contains the same coordinates as the specified 

    System.Object.

  

  

   obj: The System.Object to test.

   Returns: This method returns true if obj is a System.Drawing.PointF and has the same coordinates as this 

    System.Drawing.Point.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: PointF) -> int

  

   Returns a hash code for this System.Drawing.PointF structure.

   Returns: An integer value that specifies a hash value for this System.Drawing.PointF structure.
  """
  pass
 @staticmethod
 def Subtract(pt,sz):
  """
  Subtract(pt: PointF,sz: SizeF) -> PointF

  

   Translates a System.Drawing.PointF by the negative of a specified size.

  

   pt: The System.Drawing.PointF to translate.

   sz: The System.Drawing.SizeF that specifies the numbers to subtract from the coordinates of pt.

   Returns: The translated System.Drawing.PointF.

  Subtract(pt: PointF,sz: Size) -> PointF

  

   Translates a System.Drawing.PointF by the negative of a specified size.

  

   pt: The System.Drawing.PointF to translate.

   sz: The System.Drawing.Size that specifies the numbers to subtract from the coordinates of pt.

   Returns: The translated System.Drawing.PointF.
  """
  pass
 def ToString(self):
  """
  ToString(self: PointF) -> str

  

   Converts this System.Drawing.PointF to a human readable string.

   Returns: A string that represents this System.Drawing.PointF.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+yx.__add__(y) <==> x+y """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,x,y):
  """
  __new__[PointF]() -> PointF

  

  __new__(cls: type,x: Single,y: Single)
  """
  pass
 def __ne__(self,*args):
  pass
 def __sub__(self,*args):
  """ x.__sub__(y) <==> x-yx.__sub__(y) <==> x-y """
  pass
 IsEmpty=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether this System.Drawing.PointF is empty.



Get: IsEmpty(self: PointF) -> bool



"""

 X=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the x-coordinate of this System.Drawing.PointF.



Get: X(self: PointF) -> Single



Set: X(self: PointF)=value

"""

 Y=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the y-coordinate of this System.Drawing.PointF.



Get: Y(self: PointF) -> Single



Set: Y(self: PointF)=value

"""


 Empty=None




class LinkLabelLinkClickedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.LinkLabel.LinkClicked event.

 

 LinkLabelLinkClickedEventArgs(link: Link)

 LinkLabelLinkClickedEventArgs(link: Link,button: MouseButtons)
 """
 @staticmethod
 def __new__(self,link,button=None):
  """
  __new__(cls: type,link: Link)

  __new__(cls: type,link: Link,button: MouseButtons)
  """
  pass
 Button=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the mouse button that causes the link to be clicked.



Get: Button(self: LinkLabelLinkClickedEventArgs) -> MouseButtons



"""

 Link=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.LinkLabel.Link that was clicked.



Get: Link(self: LinkLabelLinkClickedEventArgs) -> Link



"""





# encoding: utf-8
# module System.Windows.Controls calls itself Controls
# from PresentationFramework,Version=4.0.0.0,Culture=neutral,PublicKeyToken=31bf3856ad364e35
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes
from __init___parts.AccessText import AccessText
from __init___parts.AddingNewItemEventArgs import AddingNewItemEventArgs
from __init___parts.AdornedElementPlaceholder import AdornedElementPlaceholder
from __init___parts.AlternationConverter import AlternationConverter
from __init___parts.BooleanToVisibilityConverter import BooleanToVisibilityConverter
from __init___parts.Decorator import Decorator
from __init___parts.Border import Border
from __init___parts.BorderGapMaskConverter import BorderGapMaskConverter
from __init___parts.Control import Control
from __init___parts.ContentControl import ContentControl
from __init___parts.Button import Button
from __init___parts.Calendar import Calendar
from __init___parts.CalendarBlackoutDatesCollection import CalendarBlackoutDatesCollection
from __init___parts.CalendarDateChangedEventArgs import CalendarDateChangedEventArgs
from __init___parts.CalendarDateRange import CalendarDateRange
from __init___parts.CalendarMode import CalendarMode
from __init___parts.CalendarModeChangedEventArgs import CalendarModeChangedEventArgs
from __init___parts.CalendarSelectionMode import CalendarSelectionMode
from __init___parts.Panel import Panel
from __init___parts.Canvas import Canvas
from __init___parts.CharacterCasing import CharacterCasing
from __init___parts.CheckBox import CheckBox
from __init___parts.CleanUpVirtualizedItemEventArgs import CleanUpVirtualizedItemEventArgs
from __init___parts.CleanUpVirtualizedItemEventHandler import CleanUpVirtualizedItemEventHandler
from __init___parts.ClickMode import ClickMode
from __init___parts.DefinitionBase import DefinitionBase
from __init___parts.ColumnDefinition import ColumnDefinition
from __init___parts.ColumnDefinitionCollection import ColumnDefinitionCollection
from __init___parts.ItemsControl import ItemsControl
from __init___parts.ComboBox import ComboBox
from __init___parts.ListBoxItem import ListBoxItem
from __init___parts.ComboBoxItem import ComboBoxItem
from __init___parts.ContentPresenter import ContentPresenter
from __init___parts.ContextMenu import ContextMenu
from __init___parts.ContextMenuEventArgs import ContextMenuEventArgs
from __init___parts.ContextMenuEventHandler import ContextMenuEventHandler
from __init___parts.ContextMenuService import ContextMenuService
from __init___parts.ControlTemplate import ControlTemplate
from __init___parts.ValidationRule import ValidationRule
from __init___parts.DataErrorValidationRule import DataErrorValidationRule
from __init___parts.DataGrid import DataGrid
from __init___parts.DataGridAutoGeneratingColumnEventArgs import DataGridAutoGeneratingColumnEventArgs
from __init___parts.DataGridBeginningEditEventArgs import DataGridBeginningEditEventArgs
from __init___parts.DataGridColumn import DataGridColumn
from __init___parts.DataGridBoundColumn import DataGridBoundColumn
from __init___parts.DataGridCell import DataGridCell
from __init___parts.DataGridCellClipboardEventArgs import DataGridCellClipboardEventArgs
from __init___parts.DataGridCellEditEndingEventArgs import DataGridCellEditEndingEventArgs
from __init___parts.DataGridCellInfo import DataGridCellInfo
from __init___parts.VirtualizingPanel import VirtualizingPanel
from __init___parts.DataGridCellsPanel import DataGridCellsPanel
from __init___parts.DataGridCheckBoxColumn import DataGridCheckBoxColumn
from __init___parts.DataGridClipboardCellContent import DataGridClipboardCellContent
from __init___parts.DataGridClipboardCopyMode import DataGridClipboardCopyMode
from __init___parts.DataGridColumnEventArgs import DataGridColumnEventArgs
from __init___parts.DataGridColumnReorderingEventArgs import DataGridColumnReorderingEventArgs
from __init___parts.DataGridComboBoxColumn import DataGridComboBoxColumn
from __init___parts.DataGridEditAction import DataGridEditAction
from __init___parts.DataGridEditingUnit import DataGridEditingUnit
from __init___parts.DataGridGridLinesVisibility import DataGridGridLinesVisibility
from __init___parts.DataGridHeadersVisibility import DataGridHeadersVisibility
from __init___parts.DataGridHyperlinkColumn import DataGridHyperlinkColumn
from __init___parts.DataGridLength import DataGridLength
from __init___parts.DataGridLengthConverter import DataGridLengthConverter
from __init___parts.DataGridLengthUnitType import DataGridLengthUnitType
from __init___parts.DataGridPreparingCellForEditEventArgs import DataGridPreparingCellForEditEventArgs
from __init___parts.DataGridRow import DataGridRow
from __init___parts.DataGridRowClipboardEventArgs import DataGridRowClipboardEventArgs
from __init___parts.DataGridRowDetailsEventArgs import DataGridRowDetailsEventArgs
from __init___parts.DataGridRowDetailsVisibilityMode import DataGridRowDetailsVisibilityMode
from __init___parts.DataGridRowEditEndingEventArgs import DataGridRowEditEndingEventArgs
from __init___parts.DataGridRowEventArgs import DataGridRowEventArgs
from __init___parts.DataGridSelectionMode import DataGridSelectionMode
from __init___parts.DataGridSelectionUnit import DataGridSelectionUnit
from __init___parts.DataGridSortingEventArgs import DataGridSortingEventArgs
from __init___parts.DataGridSortingEventHandler import DataGridSortingEventHandler
from __init___parts.DataGridTemplateColumn import DataGridTemplateColumn
from __init___parts.DataGridTextColumn import DataGridTextColumn
from __init___parts.DataTemplateSelector import DataTemplateSelector
from __init___parts.DatePicker import DatePicker
from __init___parts.DatePickerDateValidationErrorEventArgs import DatePickerDateValidationErrorEventArgs
from __init___parts.DatePickerFormat import DatePickerFormat
from __init___parts.Dock import Dock
from __init___parts.DockPanel import DockPanel
from __init___parts.DocumentViewer import DocumentViewer
from __init___parts.ExceptionValidationRule import ExceptionValidationRule
from __init___parts.ExpandDirection import ExpandDirection
from __init___parts.HeaderedContentControl import HeaderedContentControl
from __init___parts.Expander import Expander
from __init___parts.FlowDocumentPageViewer import FlowDocumentPageViewer
from __init___parts.FlowDocumentReader import FlowDocumentReader
from __init___parts.FlowDocumentReaderViewingMode import FlowDocumentReaderViewingMode
from __init___parts.FlowDocumentScrollViewer import FlowDocumentScrollViewer
from __init___parts.Frame import Frame
from __init___parts.Grid import Grid
from __init___parts.GridResizeBehavior import GridResizeBehavior
from __init___parts.GridResizeDirection import GridResizeDirection
from __init___parts.GridSplitter import GridSplitter
from __init___parts.ViewBase import ViewBase
from __init___parts.GridView import GridView
from __init___parts.GridViewColumn import GridViewColumn
from __init___parts.GridViewColumnCollection import GridViewColumnCollection
from __init___parts.GridViewColumnHeader import GridViewColumnHeader
from __init___parts.GridViewColumnHeaderRole import GridViewColumnHeaderRole
from __init___parts.GridViewHeaderRowPresenter import GridViewHeaderRowPresenter
from __init___parts.GridViewRowPresenter import GridViewRowPresenter
from __init___parts.GroupBox import GroupBox
from __init___parts.GroupItem import GroupItem
from __init___parts.GroupStyle import GroupStyle
from __init___parts.GroupStyleSelector import GroupStyleSelector
from __init___parts.HeaderedItemsControl import HeaderedItemsControl
from __init___parts.HierarchicalVirtualizationConstraints import HierarchicalVirtualizationConstraints
from __init___parts.HierarchicalVirtualizationHeaderDesiredSizes import HierarchicalVirtualizationHeaderDesiredSizes
from __init___parts.HierarchicalVirtualizationItemDesiredSizes import HierarchicalVirtualizationItemDesiredSizes
from __init___parts.Image import Image
from __init___parts.InitializingNewItemEventArgs import InitializingNewItemEventArgs
from __init___parts.InitializingNewItemEventHandler import InitializingNewItemEventHandler
from __init___parts.InkCanvas import InkCanvas
from __init___parts.InkCanvasClipboardFormat import InkCanvasClipboardFormat
from __init___parts.InkCanvasEditingMode import InkCanvasEditingMode
from __init___parts.InkCanvasGestureEventArgs import InkCanvasGestureEventArgs
from __init___parts.InkCanvasGestureEventHandler import InkCanvasGestureEventHandler
from __init___parts.InkCanvasSelectionChangingEventArgs import InkCanvasSelectionChangingEventArgs
from __init___parts.InkCanvasSelectionChangingEventHandler import InkCanvasSelectionChangingEventHandler
from __init___parts.InkCanvasSelectionEditingEventArgs import InkCanvasSelectionEditingEventArgs
from __init___parts.InkCanvasSelectionEditingEventHandler import InkCanvasSelectionEditingEventHandler
from __init___parts.InkCanvasSelectionHitResult import InkCanvasSelectionHitResult
from __init___parts.InkCanvasStrokeCollectedEventArgs import InkCanvasStrokeCollectedEventArgs
from __init___parts.InkCanvasStrokeCollectedEventHandler import InkCanvasStrokeCollectedEventHandler
from __init___parts.InkCanvasStrokeErasingEventArgs import InkCanvasStrokeErasingEventArgs
from __init___parts.InkCanvasStrokeErasingEventHandler import InkCanvasStrokeErasingEventHandler
from __init___parts.InkCanvasStrokesReplacedEventArgs import InkCanvasStrokesReplacedEventArgs
from __init___parts.InkCanvasStrokesReplacedEventHandler import InkCanvasStrokesReplacedEventHandler
from __init___parts.InkPresenter import InkPresenter
from __init___parts.ItemCollection import ItemCollection
from __init___parts.ItemContainerGenerator import ItemContainerGenerator
from __init___parts.ItemContainerTemplate import ItemContainerTemplate
from __init___parts.ItemContainerTemplateKey import ItemContainerTemplateKey
from __init___parts.ItemContainerTemplateSelector import ItemContainerTemplateSelector
from __init___parts.ItemsPanelTemplate import ItemsPanelTemplate
from __init___parts.ItemsPresenter import ItemsPresenter
from __init___parts.Label import Label
from __init___parts.ListBox import ListBox
from __init___parts.ListView import ListView
from __init___parts.ListViewItem import ListViewItem
from __init___parts.MediaElement import MediaElement
from __init___parts.MediaState import MediaState
from __init___parts.Menu import Menu
from __init___parts.MenuItem import MenuItem
from __init___parts.MenuItemRole import MenuItemRole
from __init___parts.MenuScrollingVisibilityConverter import MenuScrollingVisibilityConverter
from __init___parts.NotifyDataErrorValidationRule import NotifyDataErrorValidationRule
from __init___parts.Orientation import Orientation
from __init___parts.OverflowMode import OverflowMode
from __init___parts.Page import Page
from __init___parts.PageRange import PageRange
from __init___parts.PageRangeSelection import PageRangeSelection
from __init___parts.PanningMode import PanningMode
from __init___parts.PasswordBox import PasswordBox
from __init___parts.PrintDialog import PrintDialog
from __init___parts.PrintDialogException import PrintDialogException
from __init___parts.ProgressBar import ProgressBar
from __init___parts.RadioButton import RadioButton
from __init___parts.RichTextBox import RichTextBox
from __init___parts.RowDefinition import RowDefinition
from __init___parts.RowDefinitionCollection import RowDefinitionCollection
from __init___parts.ScrollBarVisibility import ScrollBarVisibility
from __init___parts.ScrollChangedEventArgs import ScrollChangedEventArgs
from __init___parts.ScrollChangedEventHandler import ScrollChangedEventHandler
from __init___parts.ScrollContentPresenter import ScrollContentPresenter
from __init___parts.ScrollUnit import ScrollUnit
from __init___parts.ScrollViewer import ScrollViewer
from __init___parts.SelectedCellsChangedEventArgs import SelectedCellsChangedEventArgs
from __init___parts.SelectedCellsChangedEventHandler import SelectedCellsChangedEventHandler
from __init___parts.SelectedDatesCollection import SelectedDatesCollection
from __init___parts.SelectionChangedEventArgs import SelectionChangedEventArgs
from __init___parts.SelectionChangedEventHandler import SelectionChangedEventHandler
from __init___parts.SelectionMode import SelectionMode
from __init___parts.SelectiveScrollingOrientation import SelectiveScrollingOrientation
from __init___parts.Separator import Separator
from __init___parts.Slider import Slider
from __init___parts.SoundPlayerAction import SoundPlayerAction
from __init___parts.SpellCheck import SpellCheck
from __init___parts.SpellingError import SpellingError
from __init___parts.SpellingReform import SpellingReform
from __init___parts.StackPanel import StackPanel
from __init___parts.StickyNoteControl import StickyNoteControl
from __init___parts.StickyNoteType import StickyNoteType
from __init___parts.StretchDirection import StretchDirection
from __init___parts.StyleSelector import StyleSelector
from __init___parts.TabControl import TabControl
from __init___parts.TabItem import TabItem
from __init___parts.TextBlock import TextBlock
from __init___parts.TextBox import TextBox
from __init___parts.TextChange import TextChange
from __init___parts.TextChangedEventArgs import TextChangedEventArgs
from __init___parts.TextChangedEventHandler import TextChangedEventHandler
from __init___parts.TextSearch import TextSearch
from __init___parts.ToolBar import ToolBar
from __init___parts.ToolBarTray import ToolBarTray
from __init___parts.ToolTip import ToolTip
from __init___parts.ToolTipEventArgs import ToolTipEventArgs
from __init___parts.ToolTipEventHandler import ToolTipEventHandler
from __init___parts.ToolTipService import ToolTipService
from __init___parts.TreeView import TreeView
from __init___parts.TreeViewItem import TreeViewItem
from __init___parts.UIElementCollection import UIElementCollection
from __init___parts.UndoAction import UndoAction
from __init___parts.UserControl import UserControl
from __init___parts.Validation import Validation
from __init___parts.ValidationError import ValidationError
from __init___parts.ValidationErrorEventAction import ValidationErrorEventAction
from __init___parts.ValidationErrorEventArgs import ValidationErrorEventArgs
from __init___parts.ValidationResult import ValidationResult
from __init___parts.ValidationStep import ValidationStep
from __init___parts.Viewbox import Viewbox
from __init___parts.Viewport3D import Viewport3D
from __init___parts.VirtualizationCacheLength import VirtualizationCacheLength
from __init___parts.VirtualizationCacheLengthConverter import VirtualizationCacheLengthConverter
from __init___parts.VirtualizationCacheLengthUnit import VirtualizationCacheLengthUnit
from __init___parts.VirtualizationMode import VirtualizationMode
from __init___parts.VirtualizingStackPanel import VirtualizingStackPanel
from __init___parts.WebBrowser import WebBrowser
from __init___parts.WrapPanel import WrapPanel



class EventLogEntry(Component,IComponent,IDisposable,ISerializable):
 """ Encapsulates a single record in the event log. This class cannot be inherited. """
 def Dispose(self):
  """
  Dispose(self: Component,disposing: bool)

   Releases the unmanaged resources used by the System.ComponentModel.Component and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def Equals(self,*__args):
  """
  Equals(self: EventLogEntry,otherEntry: EventLogEntry) -> bool

  

   Performs a comparison between two event log entries.

  

   otherEntry: The System.Diagnostics.EventLogEntry to compare.

   Returns: true if the System.Diagnostics.EventLogEntry objects are identical; otherwise,false.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 Category=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text associated with the System.Diagnostics.EventLogEntry.CategoryNumber property for this entry.



Get: Category(self: EventLogEntry) -> str



"""

 CategoryNumber=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the category number of the event log entry.



Get: CategoryNumber(self: EventLogEntry) -> Int16



"""

 Data=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the binary data associated with the entry.



Get: Data(self: EventLogEntry) -> Array[Byte]



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 EntryType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the event type of this entry.



Get: EntryType(self: EventLogEntry) -> EventLogEntryType



"""

 EventID=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the application-specific event identifier for the current event entry.



Get: EventID(self: EventLogEntry) -> int



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Index=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of this entry in the event log.



Get: Index(self: EventLogEntry) -> int



"""

 InstanceId=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the resource identifier that designates the message text of the event entry.



Get: InstanceId(self: EventLogEntry) -> Int64



"""

 MachineName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the computer on which this entry was generated.



Get: MachineName(self: EventLogEntry) -> str



"""

 Message=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the localized message associated with this event entry.



Get: Message(self: EventLogEntry) -> str



"""

 ReplacementStrings=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the replacement strings associated with the event log entry.



Get: ReplacementStrings(self: EventLogEntry) -> Array[str]



"""

 Source=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the application that generated this event.



Get: Source(self: EventLogEntry) -> str



"""

 TimeGenerated=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the local time at which this event was generated.



Get: TimeGenerated(self: EventLogEntry) -> DateTime



"""

 TimeWritten=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the local time at which this event was written to the log.



Get: TimeWritten(self: EventLogEntry) -> DateTime



"""

 UserName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the user who is responsible for this event.



Get: UserName(self: EventLogEntry) -> str



"""





class DataGridViewCellBorderStyle(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the border styles that can be applied to the cells of a System.Windows.Forms.DataGridView control.

 

 enum DataGridViewCellBorderStyle,values: Custom (0),None (4),Raised (2),RaisedHorizontal (9),RaisedVertical (6),Single (1),SingleHorizontal (8),SingleVertical (5),Sunken (3),SunkenHorizontal (10),SunkenVertical (7)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Custom=None
 None=None
 Raised=None
 RaisedHorizontal=None
 RaisedVertical=None
 Single=None
 SingleHorizontal=None
 SingleVertical=None
 Sunken=None
 SunkenHorizontal=None
 SunkenVertical=None
 value__=None




# -*- coding: utf-8 -*-
"""DirectContext3DServer wrapper and utilities."""

from collections import namedtuple
import traceback

from pyrevit import DB
from pyrevit.api import ExternalService as es
from pyrevit.api import DirectContext3D as dc3d
from pyrevit.coreutils import Guid
from pyrevit.interop import stl

Edge = namedtuple("Edge", ["a", "b", "color"])
Triangle = namedtuple("Triangle", ["a", "b", "c", "normal", "color"])


class Mesh(object):
    """Geometry container class to generate DirectContext3DServer buffers."""

    def __init__(self, edges, triangles):
        if not isinstance(edges, list):
            raise ValueError("List of Edges should be provided")
        self.edges = edges
        if not isinstance(triangles, list):
            raise ValueError("List of Triangles should be provided")
        self.triangles = triangles

        vertices = []
        for edge in self.edges:
            vertices.extend([edge.a, edge.b])
        for triangle in self.triangles:
            vertices.extend([triangle.a, triangle.b, triangle.c])
        self.vertices = vertices

    @staticmethod
    def calculate_triangle_normal(a, b, c):
        return (c - a).CrossProduct(b - a)

    @classmethod
    def from_solid(cls, doc, solid):
        try:
            if not isinstance(solid, DB.Solid):
                raise TypeError("Provided object has to be a Revit Solid!")
            triangles = []
            edges = []
            for face in solid.Faces:
                if face.MaterialElementId == DB.ElementId.InvalidElementId:
                    # make it white if material is <By Category>
                    color = DB.ColorWithTransparency(255, 255, 255, 0)
                else:
                    material = doc.GetElement(face.MaterialElementId)
                    color = DB.ColorWithTransparency(
                        material.Color.Red,
                        material.Color.Green,
                        material.Color.Blue,
                        int(material.Transparency / 100 * 255),
                    )
                face_mesh = face.Triangulate()
                triangle_count = face_mesh.NumTriangles
                for idx in range(triangle_count):
                    mesh_triangle = face_mesh.get_Triangle(idx)
                    if (
                        face_mesh.DistributionOfNormals
                        == DB.DistributionOfNormals.OnePerFace
                    ):
                        normal = face_mesh.GetNormal(0)
                    elif (
                        face_mesh.DistributionOfNormals
                        == DB.DistributionOfNormals.OnEachFacet
                    ):
                        normal = face_mesh.GetNormal(idx)
                    elif (
                        face_mesh.DistributionOfNormals
                        == DB.DistributionOfNormals.AtEachPoint
                    ):
                        normal = (
                            face_mesh.GetNormal(mesh_triangle.get_Index(0))
                            + face_mesh.GetNormal(mesh_triangle.get_Index(1))
                            + face_mesh.GetNormal(mesh_triangle.get_Index(2))
                        ).Normalize()
                    triangles.append(
                        Triangle(
                            mesh_triangle.get_Vertex(0),
                            mesh_triangle.get_Vertex(1),
                            mesh_triangle.get_Vertex(2),
                            normal,
                            color,
                        )
                    )
            for edge in solid.Edges:
                pts = edge.Tessellate()
                for idx in range(len(pts) - 1):
                    edges.append(
                        Edge(
                            pts[idx], pts[idx + 1], DB.ColorWithTransparency(0, 0, 0, 0)
                        )
                    )
            return cls(edges, triangles)
        except:
            print(traceback.format_exc())

    @classmethod
    def from_boundingbox(cls, bounding_box, color, black_edges=True):
        """Generates a Mesh object from the input BoundigBoxXYZ.

        Args:
            bounding_box (DB.BoundingBoxXYZ): The source object.
            color (DB.ColorWithTransparency): The desired color.
            black_edges (bool, optional):
                To have black edges instead of the provided color.
                Defaults to True.

        Returns:
            (Mesh): The resulting Mesh
        """
        try:
            # get all untransformed vertices of the boundingbox
            vertices = [
                [
                    DB.XYZ(bounding_box.Min.X, bounding_box.Min.Y, bounding_box.Min.Z),
                    DB.XYZ(bounding_box.Min.X, bounding_box.Max.Y, bounding_box.Min.Z),
                    DB.XYZ(bounding_box.Max.X, bounding_box.Max.Y, bounding_box.Min.Z),
                    DB.XYZ(bounding_box.Max.X, bounding_box.Min.Y, bounding_box.Min.Z),
                ],
                [
                    DB.XYZ(bounding_box.Min.X, bounding_box.Min.Y, bounding_box.Max.Z),
                    DB.XYZ(bounding_box.Min.X, bounding_box.Max.Y, bounding_box.Max.Z),
                    DB.XYZ(bounding_box.Max.X, bounding_box.Max.Y, bounding_box.Max.Z),
                    DB.XYZ(bounding_box.Max.X, bounding_box.Min.Y, bounding_box.Max.Z),
                ],
            ]
            # apply it's transform if it has any
            vertices = [
                [bounding_box.Transform.OfPoint(pt) for pt in vertices[idx]]
                for idx in range(len(vertices))
            ]

            if black_edges:
                edge_color = DB.ColorWithTransparency(0, 0, 0, 0)
            else:
                edge_color = color

            edges = []
            # creating top and bottom face triangles
            triangles = [
                Triangle(
                    vertices[0][0],
                    vertices[0][2],
                    vertices[0][1],
                    Mesh.calculate_triangle_normal(
                        vertices[0][0], vertices[0][2], vertices[0][1]
                    ),
                    color,
                ),
                Triangle(
                    vertices[0][0],
                    vertices[0][3],
                    vertices[0][2],
                    Mesh.calculate_triangle_normal(
                        vertices[0][0], vertices[0][3], vertices[0][2]
                    ),
                    color,
                ),
                Triangle(
                    vertices[1][0],
                    vertices[1][1],
                    vertices[1][2],
                    Mesh.calculate_triangle_normal(
                        vertices[1][0], vertices[1][1], vertices[1][2]
                    ),
                    color,
                ),
                Triangle(
                    vertices[1][0],
                    vertices[1][2],
                    vertices[1][3],
                    Mesh.calculate_triangle_normal(
                        vertices[1][0], vertices[1][2], vertices[1][3]
                    ),
                    color,
                ),
            ]

            # creating edges and face triangles for the sides
            for idx in range(4):
                edges.extend(
                    [
                        Edge(vertices[0][idx], vertices[1][idx], edge_color),
                        Edge(vertices[0][idx], vertices[0][idx - 1], edge_color),
                        Edge(vertices[1][idx], vertices[1][idx - 1], edge_color),
                    ]
                )
                triangles.extend(
                    [
                        Triangle(
                            vertices[0][idx],
                            vertices[1][idx],
                            vertices[0][idx - 1],
                            Mesh.calculate_triangle_normal(
                                vertices[0][idx], vertices[1][idx], vertices[0][idx - 1]
                            ),
                            color,
                        ),
                        Triangle(
                            vertices[1][idx],
                            vertices[1][idx - 1],
                            vertices[0][idx - 1],
                            Mesh.calculate_triangle_normal(
                                vertices[1][idx],
                                vertices[1][idx - 1],
                                vertices[0][idx - 1],
                            ),
                            color,
                        ),
                    ]
                )

            return cls(edges, triangles)
        except Exception:
            print(traceback.format_exc())

    @classmethod
    def from_stl(cls, stl_filepath, color, transform=None, black_edges=True):
        """Generates a Mesh object from an STL file.

        Args:
            stl_filepath (str): Path to the STL file.
            color (DB.ColorWithTransparency): The desired color.
            transform (DB.Transform, optional):
                Transform to apply to the mesh (for positioning, scaling, rotation).
                Defaults to None (identity transform).
            black_edges (bool, optional):
                To have black edges instead of the provided color.
                Defaults to True.

        Returns:
            (Mesh): The resulting Mesh
        """
        try:
            stl_mesh = stl.load(stl_filepath)

            if not stl_mesh or not stl_mesh.triangles:
                raise ValueError("STL file contains no triangles")

            edges = []
            triangles = []

            if black_edges:
                edge_color = DB.ColorWithTransparency(0, 0, 0, 0)
            else:
                edge_color = color

            for stl_triangle in stl_mesh.triangles:
                v1_tuple = stl_triangle["vertices"][0]
                v2_tuple = stl_triangle["vertices"][1]
                v3_tuple = stl_triangle["vertices"][2]
                n_tuple = stl_triangle["normal"]

                v1 = DB.XYZ(v1_tuple[0], v1_tuple[1], v1_tuple[2])
                v2 = DB.XYZ(v2_tuple[0], v2_tuple[1], v2_tuple[2])
                v3 = DB.XYZ(v3_tuple[0], v3_tuple[1], v3_tuple[2])
                normal = DB.XYZ(n_tuple[0], n_tuple[1], n_tuple[2])

                if transform is not None:
                    v1 = transform.OfPoint(v1)
                    v2 = transform.OfPoint(v2)
                    v3 = transform.OfPoint(v3)
                    normal = transform.OfVector(normal)

                try:
                    normal = normal.Normalize()
                except:
                    normal = cls.calculate_triangle_normal(v1, v2, v3)
                    try:
                        normal = normal.Normalize()
                    except:
                        normal = DB.XYZ(0, 0, 1)

                triangles.append(Triangle(v1, v2, v3, normal, color))

                edges.append(Edge(v1, v2, edge_color))
                edges.append(Edge(v2, v3, edge_color))
                edges.append(Edge(v3, v1, edge_color))

            return cls(edges, triangles)

        except Exception:
            print(traceback.format_exc())
            return None


class Buffer(object):
    """Buffer container for DirectContext3DServer."""

    def __init__(
        self,
        display_style,
        vertex_buffer,
        vertex_count,
        index_buffer,
        index_count,
        vertex_format,
        effect_instance,
        primitive_type,
        start,
        primitive_count,
    ):
        self.display_style = display_style
        self.vertex_buffer = vertex_buffer
        self.vertex_count = vertex_count
        self.index_buffer = index_buffer
        self.index_count = index_count
        self.vertex_format = vertex_format
        self.effect_instance = effect_instance
        self.primitive_type = primitive_type
        self.start = start
        self.primitive_count = primitive_count

    def is_valid(self, display_style):
        return all(
            [
                self.display_style == display_style,
                self.vertex_buffer.IsValid,
                self.index_buffer.IsValid,
                self.vertex_format.IsValid,
                self.effect_instance.IsValid,
            ]
        )

    @classmethod
    def build_edge_buffer(cls, display_style, meshes):
        try:
            if not meshes:
                return
            edges = [edge for edges in [m.edges for m in meshes] for edge in edges]

            format_bits = dc3d.VertexFormatBits.PositionColored
            vertex_format = dc3d.VertexFormat(format_bits)

            effect_instance = dc3d.EffectInstance(format_bits)

            vertex_count = dc3d.VertexPositionColored.GetSizeInFloats() * len(edges) * 2
            vertex_buffer = dc3d.VertexBuffer(vertex_count)

            index_count = dc3d.IndexLine.GetSizeInShortInts() * len(edges)
            index_buffer = dc3d.IndexBuffer(index_count)

            vertex_buffer.Map(vertex_count)
            index_buffer.Map(index_count)
            vertex_stream_pos = vertex_buffer.GetVertexStreamPositionColored()
            index_stream_pos = index_buffer.GetIndexStreamLine()
            for edge_index, edge in enumerate(edges):
                first_idx = edge_index * 2
                vertex_stream_pos.AddVertex(
                    dc3d.VertexPositionColored(edge.a, edge.color)
                )
                vertex_stream_pos.AddVertex(
                    dc3d.VertexPositionColored(edge.b, edge.color)
                )
                index_stream_pos.AddLine(dc3d.IndexLine(first_idx, first_idx + 1))
            vertex_buffer.Unmap()
            index_buffer.Unmap()

            return cls(
                display_style,
                vertex_buffer,
                vertex_count,
                index_buffer,
                index_count,
                vertex_format,
                effect_instance,
                dc3d.PrimitiveType.LineList,
                0,
                len(edges),
            )
        except:
            print(traceback.format_exc())

    @classmethod
    def build_triangle_buffer(cls, display_style, meshes, transparent=False):
        try:
            if not meshes:
                return
            triangles = [
                triangle
                for triangles in [m.triangles for m in meshes]
                for triangle in triangles
                if transparent == (triangle.color.GetTransparency() > 0)
            ]

            if not triangles:
                return

            shaded = any(
                [
                    display_style == DB.DisplayStyle.Shading,
                    display_style == DB.DisplayStyle.ShadingWithEdges,
                ]
            )

            format_bits = dc3d.VertexFormatBits.PositionNormalColored
            vertex_format = dc3d.VertexFormat(format_bits)

            if shaded:
                effect_instance = dc3d.EffectInstance(
                    dc3d.VertexFormatBits.PositionNormalColored
                )
            else:
                effect_instance = dc3d.EffectInstance(
                    dc3d.VertexFormatBits.PositionColored
                )

            vertex_count = (
                dc3d.VertexPositionNormalColored.GetSizeInFloats() * len(triangles) * 3
            )
            vertex_buffer = dc3d.VertexBuffer(vertex_count)

            index_count = dc3d.IndexTriangle.GetSizeInShortInts() * len(triangles)
            index_buffer = dc3d.IndexBuffer(index_count)

            vertex_buffer.Map(vertex_count)
            index_buffer.Map(index_count)
            vertex_stream_pos = vertex_buffer.GetVertexStreamPositionNormalColored()
            index_stream_pos = index_buffer.GetIndexStreamTriangle()
            for triangle_index, triangle in enumerate(triangles):
                if display_style == DB.DisplayStyle.HLR:
                    color = DB.ColorWithTransparency(
                        255, 255, 255, triangle.color.GetTransparency()
                    )
                else:
                    color = triangle.color
                normal = triangle.normal
                first_idx = triangle_index * 3
                vertex_stream_pos.AddVertex(
                    dc3d.VertexPositionNormalColored(triangle.a, normal, color)
                )
                vertex_stream_pos.AddVertex(
                    dc3d.VertexPositionNormalColored(triangle.b, normal, color)
                )
                vertex_stream_pos.AddVertex(
                    dc3d.VertexPositionNormalColored(triangle.c, normal, color)
                )
                index_stream_pos.AddTriangle(
                    dc3d.IndexTriangle(first_idx, first_idx + 1, first_idx + 2)
                )
            vertex_buffer.Unmap()
            index_buffer.Unmap()

            return cls(
                display_style,
                vertex_buffer,
                vertex_count,
                index_buffer,
                index_count,
                vertex_format,
                effect_instance,
                dc3d.PrimitiveType.TriangleList,
                0,
                len(triangles),
            )
        except:
            print(traceback.format_exc())

    def flush(self):
        dc3d.DrawContext.FlushBuffer(
            self.vertex_buffer,
            self.vertex_count,
            self.index_buffer,
            self.index_count,
            self.vertex_format,
            self.effect_instance,
            self.primitive_type,
            self.start,
            self.primitive_count,
        )


class Server(dc3d.IDirectContext3DServer):
    """A generic DirectContext3dServer interface implementation."""

    def __init__(
        self,
        guid=None,
        uidoc=None,
        name="pyRevit DirectContext3DServer",
        description="Displaying temporary geometries",
        vendor_id="pyRevit",
        register=True,
    ):
        """Initalize a DirectContext3DServer instance.

        Args:
            guid (System.Guid, optional):
                The guid of the server, auto-generated if None.
                Defaults to None.
            uidoc (UI.UIDocument, optional):
                Required only, if the geometry is Document specific.
                Defaults to None.
            name (str, optional):
                The name of the server.
                Defaults to "pyRevit DirectContext3DServer".
            description (str, optional):
                The descritpion of the server.
                Defaults to "Displaying temporary geometries".
            vendor_id (str, optional):
                The name of the vendor. Defaults to "pyRevit".
            register (bool, optional):
                To register the server on initalization.
                Defaults to True.
        """
        try:
            if guid is None:
                self.guid = Guid.NewGuid()
            else:
                self.guid = guid
            self.uidoc = uidoc
            self.name = name
            self.description = description
            self.vendor_id = vendor_id
            self.meshes = []
            self._line_buffer = None
            self._transparent_buffer = None
            self._opaque_buffer = None
            self.enabled_view_types = [
                DB.ViewType.ThreeD,
                DB.ViewType.Elevation,
                DB.ViewType.Section,
                DB.ViewType.FloorPlan,
                DB.ViewType.CeilingPlan,
                DB.ViewType.EngineeringPlan,
            ]

            if register:
                self.add_server()

        except Exception:
            print(traceback.format_exc())

    # region Interface methods
    def CanExecute(self, view):
        if any(vt == view.ViewType for vt in self.enabled_view_types):
            if self.uidoc is not None:
                return self.uidoc.Document.Equals(view.Document)
            return True
        return False

    def GetApplicationId(self):
        return ""

    def GetBoundingBox(self, view):
        try:
            if not self.meshes:
                return

            vertices = [
                vertex
                for vertices in [m.vertices for m in self.meshes]
                for vertex in vertices
            ]

            if not len(vertices) > 1:
                return

            outline = DB.Outline(vertices[0], vertices[1])

            for idx in range(2, len(vertices)):
                outline.AddPoint(vertices[idx])

            return outline
        except:
            print(traceback.format_exc())

    def GetDescription(self):
        return self.description

    def GetName(self):
        return self.name

    def GetServerId(self):
        return self.guid

    def GetServiceId(self):
        return es.ExternalServices.BuiltInExternalServices.DirectContext3DService

    def GetSourceId(self):
        return ""

    def GetVendorId(self):
        return self.vendor_id

    def UsesHandles(self):
        return False

    def UseInTransparentPass(self, view):
        return True

    def RenderScene(self, view, display_style):
        try:
            if self.meshes:
                if all(
                    [
                        self._line_buffer is None,
                        self._opaque_buffer is None,
                        self._transparent_buffer is None,
                    ]
                ) or any(
                    [
                        (
                            self._line_buffer is not None
                            and not self._line_buffer.is_valid(display_style)
                        ),
                        (
                            self._opaque_buffer is not None
                            and not self._opaque_buffer.is_valid(display_style)
                        ),
                        (
                            self._transparent_buffer is not None
                            and not self._transparent_buffer.is_valid(display_style)
                        ),
                    ]
                ):
                    self.update_buffers(display_style)

                if dc3d.DrawContext.IsTransparentPass():
                    if (
                        self._transparent_buffer is not None
                        and display_style != DB.DisplayStyle.Wireframe
                    ):
                        self._transparent_buffer.flush()
                else:
                    if (
                        self._opaque_buffer is not None
                        and display_style != DB.DisplayStyle.Wireframe
                    ):
                        self._opaque_buffer.flush()
                    if (
                        self._line_buffer is not None
                        and display_style != DB.DisplayStyle.Rendering
                        and display_style != DB.DisplayStyle.Shading
                    ):
                        self._line_buffer.flush()
        except:
            print(traceback.format_exc())

    # endregion

    @property
    def meshes(self):
        """The container for the source geometries."""
        return self._meshes

    @meshes.setter
    def meshes(self, meshes):
        self._meshes = meshes
        try:
            self._meshes = meshes
            self._line_buffer = None
            self._opaque_buffer = None
            self._transparent_buffer = None
        except Exception:
            print(traceback.format_exc())

    def add_server(self):
        service = es.ExternalServiceRegistry.GetService(self.GetServiceId())
        # check for leftovers
        if service.IsRegisteredServerId(self.GetServerId()):
            self.remove_server()
        service.AddServer(self)
        server_ids = service.GetActiveServerIds()
        server_ids.Add(self.GetServerId())
        service.SetActiveServers(server_ids)

    def remove_server(self):
        service = es.ExternalServiceRegistry.GetService(self.GetServiceId())
        if service.IsRegisteredServerId(self.GetServerId()):
            service.RemoveServer(self.GetServerId())

    def update_buffers(self, display_style):
        self._line_buffer = Buffer.build_edge_buffer(display_style, self.meshes)
        self._opaque_buffer = Buffer.build_triangle_buffer(
            display_style, self.meshes, transparent=False
        )
        self._transparent_buffer = Buffer.build_triangle_buffer(
            display_style, self.meshes, transparent=True
        )



class SpellingError(object):
 """ Represents a misspelled word in an editing control (i.e. System.Windows.Controls.TextBox or System.Windows.Controls.RichTextBox). """
 def Correct(self,correctedText):
  """
  Correct(self: SpellingError,correctedText: str)

   Replaces the spelling error text with the specified correction.

  

   correctedText: The text used to replace the misspelled text.
  """
  pass
 def IgnoreAll(self):
  """
  IgnoreAll(self: SpellingError)

   Instructs the control to ignore this error and any duplicates for the remainder of the lifetime 

    of the control.
  """
  pass
 Suggestions=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a list of suggested spelling replacements for the misspelled word.



Get: Suggestions(self: SpellingError) -> IEnumerable[str]



"""





"""Unit conversion utilities for Revit."""

from pyrevit import DOCS, HOST_APP
from pyrevit import DB


def format_area(area_value, doc=None):
    """Return formatted area value in document units.

    Args:
        area_value (float): area value
        doc (DB.Document, optional): Revit document, defaults to current

    Returns:
        (str): formatted value
    """
    doc = doc or DOCS.doc
    if HOST_APP.is_newer_than(2021):
        return DB.UnitFormatUtils.Format(units=doc.GetUnits(),
                                         specTypeId=DB.SpecTypeId.Area,
                                         value=area_value,
                                         forEditing=False)
    else:
        return DB.UnitFormatUtils.Format(units=doc.GetUnits(),
                                         unitType=DB.UnitType.UT_Area,
                                         value=area_value,
                                         maxAccuracy=False,
                                         forEditing=False)


def format_slope(slope_value, doc=None):
    """Return formatted slope value in document units.

    Args:
        slope_value (float): slope value
        doc (DB.Document, optional): Revit document, defaults to current

    Returns:
        (str): formatted value
    """
    doc = doc or DOCS.doc
    if HOST_APP.is_newer_than(2021):
        return DB.UnitFormatUtils.Format(units=doc.GetUnits(),
                                         specTypeId=DB.SpecTypeId.Slope,
                                         value=slope_value,
                                         forEditing=False)
    else:
        return DB.UnitFormatUtils.Format(units=doc.GetUnits(),
                                         unitType=DB.UnitType.UT_Slope,
                                         value=slope_value,
                                         maxAccuracy=False,
                                         forEditing=False)


def _create_view_plane(view):
    """Get a plane parallel to a view.

    Args:
        view (DB.View): view to align plane

    Returns:
        DB.Plane: result plane
    """
    return DB.Plane.CreateByOriginAndBasis(
        view.Origin, view.RightDirection, view.UpDirection)


def project_to_viewport(xyz, view):
    """Project a point to viewport coordinates.

    Args:
        xyz (DB.XYZ): point to project
        view (DB.View): target view

    Returns:
        (DB.UV): [description]
    """
    plane = _create_view_plane(view)
    uv, _ = plane.Project(xyz)
    return uv


def project_to_world(uv, view):
    """Get view-based point (UV) back to model coordinates.

    Args:
        uv (DB.UV): point on a view
        view (DB.View): view to get coordinates from

    Returns:
        (DB.XYZ): point in world coordinates
    """
    plane = _create_view_plane(view)
    trf = DB.Transform.Identity
    trf.BasisX = plane.XVec
    trf.BasisY = plane.YVec
    trf.BasisZ = plane.Normal
    trf.Origin = plane.Origin
    return trf.OfPoint(DB.XYZ(uv.U, uv.V, 0))


def get_spec_name(forge_id):
    """Returns the measurable spec name for the given unit id.

    Args:
        forge_id (DB.ForgeTypeId): Unit id

    Returns:
        (str): Spec name
    """
    if HOST_APP.is_newer_than(2021) \
            and DB.UnitUtils.IsMeasurableSpec(forge_id):
        return DB.UnitUtils.GetTypeCatalogStringForSpec(forge_id)
    return ""


def get_unit_name(forge_id):
    """Returns the unit name for the given unit id.

    Args:
        forge_id (DB.ForgeTypeId): Unit id

    Returns:
        (str): Unit name
    """
    if HOST_APP.is_newer_than(2021) \
            and DB.UnitUtils.IsUnit(forge_id):
        return DB.UnitUtils.GetTypeCatalogStringForUnit(forge_id)
    return ""



class IContainerControl:
 """ Provides the functionality for a control to act as a parent for other controls. """
 def ActivateControl(self,active):
  """
  ActivateControl(self: IContainerControl,active: Control) -> bool

  

   Activates a specified control.

  

   active: The System.Windows.Forms.Control being activated.

   Returns: true if the control is successfully activated; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 ActiveControl=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the control that is active on the container control.



Get: ActiveControl(self: IContainerControl) -> Control



Set: ActiveControl(self: IContainerControl)=value

"""





class FileSystemRights(Enum,IComparable,IFormattable,IConvertible):
 """
 Defines the access rights to use when creating access and audit rules.

 

 enum (flags) FileSystemRights,values: AppendData (4),ChangePermissions (262144),CreateDirectories (4),CreateFiles (2),Delete (65536),DeleteSubdirectoriesAndFiles (64),ExecuteFile (32),FullControl (2032127),ListDirectory (1),Modify (197055),Read (131209),ReadAndExecute (131241),ReadAttributes (128),ReadData (1),ReadExtendedAttributes (8),ReadPermissions (131072),Synchronize (1048576),TakeOwnership (524288),Traverse (32),Write (278),WriteAttributes (256),WriteData (2),WriteExtendedAttributes (16)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AppendData=None
 ChangePermissions=None
 CreateDirectories=None
 CreateFiles=None
 Delete=None
 DeleteSubdirectoriesAndFiles=None
 ExecuteFile=None
 FullControl=None
 ListDirectory=None
 Modify=None
 Read=None
 ReadAndExecute=None
 ReadAttributes=None
 ReadData=None
 ReadExtendedAttributes=None
 ReadPermissions=None
 Synchronize=None
 TakeOwnership=None
 Traverse=None
 value__=None
 Write=None
 WriteAttributes=None
 WriteData=None
 WriteExtendedAttributes=None




class ToolStripSeparatorRenderEventArgs(ToolStripItemRenderEventArgs):
 """
 Provides data for the System.Windows.Forms.ToolStripRenderer.RenderGrip event.

 

 ToolStripSeparatorRenderEventArgs(g: Graphics,separator: ToolStripSeparator,vertical: bool)
 """
 @staticmethod
 def __new__(self,g,separator,vertical):
  """ __new__(cls: type,g: Graphics,separator: ToolStripSeparator,vertical: bool) """
  pass
 Vertical=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the display style for the grip is vertical.



Get: Vertical(self: ToolStripSeparatorRenderEventArgs) -> bool



"""





class DataGridTextColumn(DataGridBoundColumn):
 """
 Represents a System.Windows.Controls.DataGrid column that hosts textual content in its cells.

 

 DataGridTextColumn()
 """
 DataGridOwner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Controls.DataGrid control that contains this column.



"""

 FontFamily=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the font family for the content of cells in the column.



Get: FontFamily(self: DataGridTextColumn) -> FontFamily



Set: FontFamily(self: DataGridTextColumn)=value

"""

 FontSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the font size for the content of cells in the column.



Get: FontSize(self: DataGridTextColumn) -> float



Set: FontSize(self: DataGridTextColumn)=value

"""

 FontStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the font style for the content of cells in the column.



Get: FontStyle(self: DataGridTextColumn) -> FontStyle



Set: FontStyle(self: DataGridTextColumn)=value

"""

 FontWeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the font weight for the content of cells in the column.



Get: FontWeight(self: DataGridTextColumn) -> FontWeight



Set: FontWeight(self: DataGridTextColumn)=value

"""

 Foreground=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.Brush that is used to paint the text contents of cells in the column.



Get: Foreground(self: DataGridTextColumn) -> Brush



Set: Foreground(self: DataGridTextColumn)=value

"""


 DefaultEditingElementStyle=None
 DefaultElementStyle=None
 FontFamilyProperty=None
 FontSizeProperty=None
 FontStyleProperty=None
 FontWeightProperty=None
 ForegroundProperty=None




class FileDialogCustomPlace(object):
 """
 Represents an entry in a System.Windows.Forms.FileDialog custom place collection for Windows Vista.

 

 FileDialogCustomPlace(path: str)

 FileDialogCustomPlace(knownFolderGuid: Guid)
 """
 def ToString(self):
  """
  ToString(self: FileDialogCustomPlace) -> str

  

   Returns a string that represents this System.Windows.Forms.FileDialogCustomPlace instance.

   Returns: A string that represents this System.Windows.Forms.FileDialogCustomPlace instance.
  """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,path: str)

  __new__(cls: type,knownFolderGuid: Guid)
  """
  pass
 KnownFolderGuid=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the Windows Vista Known Folder GUID for the custom place.



Get: KnownFolderGuid(self: FileDialogCustomPlace) -> Guid



Set: KnownFolderGuid(self: FileDialogCustomPlace)=value

"""

 Path=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the folder path to the custom place.



Get: Path(self: FileDialogCustomPlace) -> str



Set: Path(self: FileDialogCustomPlace)=value

"""





class LayoutSettings(object):
 """ Provides a base class for collecting layout scheme characteristics. """
 LayoutEngine=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current table layout engine.



Get: LayoutEngine(self: LayoutSettings) -> LayoutEngine



"""





class UICues(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the state of the user interface.

 

 enum (flags) UICues,values: Changed (12),ChangeFocus (4),ChangeKeyboard (8),None (0),ShowFocus (1),ShowKeyboard (2),Shown (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Changed=None
 ChangeFocus=None
 ChangeKeyboard=None
 None=None
 ShowFocus=None
 ShowKeyboard=None
 Shown=None
 value__=None




class XmlWriterTraceListener(TextWriterTraceListener,IDisposable):
 """
 Directs tracing or debugging output as XML-encoded data to a System.IO.TextWriter or to a System.IO.Stream,such as a System.IO.FileStream.

 

 XmlWriterTraceListener(stream: Stream)

 XmlWriterTraceListener(stream: Stream,name: str)

 XmlWriterTraceListener(writer: TextWriter)

 XmlWriterTraceListener(writer: TextWriter,name: str)

 XmlWriterTraceListener(filename: str)

 XmlWriterTraceListener(filename: str,name: str)
 """
 def Close(self):
  """
  Close(self: XmlWriterTraceListener)

   Closes the System.Diagnostics.TextWriterTraceListener.Writer for this listener so that it no 

    longer receives tracing or debugging output.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: TextWriterTraceListener,disposing: bool)

   Disposes this System.Diagnostics.TextWriterTraceListener object.

  

   disposing: true to release managed resources; if false,

    System.Diagnostics.TextWriterTraceListener.Dispose(System.Boolean) has no effect.
  """
  pass
 def Fail(self,message,detailMessage=None):
  """
  Fail(self: XmlWriterTraceListener,message: str,detailMessage: str)

   Writes trace information including an error message and a detailed error message to the file or 

    stream.

  

  

   message: The error message to write.

   detailMessage: The detailed error message to append to the error message.
  """
  pass
 def GetSupportedAttributes(self,*args):
  """
  GetSupportedAttributes(self: TraceListener) -> Array[str]

  

   Gets the custom attributes supported by the trace listener.

   Returns: A string array naming the custom attributes supported by the trace listener,or null if there 

    are no custom attributes.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def TraceData(self,eventCache,source,eventType,id,data):
  """
  TraceData(self: XmlWriterTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,*data: Array[object])

   Writes trace information,data objects,and event information to the file or stream.

  

   eventCache: A System.Diagnostics.TraceEventCache that contains the current process ID,thread ID,and stack 

    trace information.

  

   source: The source name.

   eventType: One of the System.Diagnostics.TraceEventType values.

   id: A numeric identifier for the event.

   data: An array of data objects to emit.

  TraceData(self: XmlWriterTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,data: object)

   Writes trace information,a data object,and event information to the file or stream.

  

   eventCache: A System.Diagnostics.TraceEventCache that contains the current process ID,thread ID,and stack 

    trace information.

  

   source: The source name.

   eventType: One of the System.Diagnostics.TraceEventType values.

   id: A numeric identifier for the event.

   data: A data object to emit.
  """
  pass
 def TraceEvent(self,eventCache,source,eventType,id,*__args):
  """
  TraceEvent(self: XmlWriterTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,message: str)

   Writes trace information,a message,and event information to the file or stream.

  

   eventCache: A System.Diagnostics.TraceEventCache that contains the current process ID,thread ID,and stack 

    trace information.

  

   source: The source name.

   eventType: One of the System.Diagnostics.TraceEventType values.

   id: A numeric identifier for the event.

   message: The message to write.

  TraceEvent(self: XmlWriterTraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,format: str,*args: Array[object])

   Writes trace information,a formatted message,and event information to the file or stream.

  

   eventCache: A System.Diagnostics.TraceEventCache that contains the current process ID,thread ID,and stack 

    trace information.

  

   source: The source name.

   eventType: One of the System.Diagnostics.TraceEventType values.

   id: A numeric identifier for the event.

   format: A format string that contains zero or more format items that correspond to objects in the args 

    array.

  

   args: An object array containing zero or more objects to format.
  """
  pass
 def TraceTransfer(self,eventCache,source,id,message,relatedActivityId):
  """
  TraceTransfer(self: XmlWriterTraceListener,eventCache: TraceEventCache,source: str,id: int,message: str,relatedActivityId: Guid)

   Writes trace information including the identity of a related activity,a message,and event 

    information to the file or stream.

  

  

   eventCache: A System.Diagnostics.TraceEventCache that contains the current process ID,thread ID,and stack 

    trace information.

  

   source: The source name.

   id: A numeric identifier for the event.

   message: A trace message to write.

   relatedActivityId: A System.Guid structure that identifies a related activity.
  """
  pass
 def Write(self,*__args):
  """
  Write(self: XmlWriterTraceListener,message: str)

   Writes a verbatim message without any additional context information to the file or stream.

  

   message: The message to write.
  """
  pass
 def WriteIndent(self,*args):
  """
  WriteIndent(self: TraceListener)

   Writes the indent to the listener you create when you implement this class,and resets the 

    System.Diagnostics.TraceListener.NeedIndent property to false.
  """
  pass
 def WriteLine(self,*__args):
  """
  WriteLine(self: XmlWriterTraceListener,message: str)

   Writes a verbatim message without any additional context information followed by the current 

    line terminator to the file or stream.

  

  

   message: The message to write.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,stream: Stream)

  __new__(cls: type,stream: Stream,name: str)

  __new__(cls: type,writer: TextWriter)

  __new__(cls: type,writer: TextWriter,name: str)

  __new__(cls: type,filename: str)

  __new__(cls: type,filename: str,name: str)
  """
  pass
 NeedIndent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether to indent the output.



"""



# variables with complex values




class ValidationErrorEventArgs(RoutedEventArgs):
 """ Provides information for the System.Windows.Controls.Validation.Error�attached event. """
 Action=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the error is a new error or an existing error that has now been cleared.



Get: Action(self: ValidationErrorEventArgs) -> ValidationErrorEventAction



"""

 Error=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the error that caused this System.Windows.Controls.Validation.Error event.



Get: Error(self: ValidationErrorEventArgs) -> ValidationError



"""





class IButtonControl:
 """ Allows a control to act like a button on a form. """
 def NotifyDefault(self,value):
  """
  NotifyDefault(self: IButtonControl,value: bool)

   Notifies a control that it is the default button so that its appearance and behavior is adjusted 

    accordingly.

  

  

   value: true if the control should behave as a default button; otherwise false.
  """
  pass
 def PerformClick(self):
  """
  PerformClick(self: IButtonControl)

   Generates a System.Windows.Forms.Control.Click event for the control.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 DialogResult=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value returned to the parent form when the button is clicked.



Get: DialogResult(self: IButtonControl) -> DialogResult



Set: DialogResult(self: IButtonControl)=value

"""





class BatteryChargeStatus(Enum,IComparable,IFormattable,IConvertible):
 """
 Defines identifiers that indicate the current battery charge level or charging state information.

 

 enum (flags) BatteryChargeStatus,values: Charging (8),Critical (4),High (1),Low (2),NoSystemBattery (128),Unknown (255)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Charging=None
 Critical=None
 High=None
 Low=None
 NoSystemBattery=None
 Unknown=None
 value__=None

